<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is released under commercial license by Lamia Oy.
 *
 * @copyright Copyright (c) 2018 Lamia Oy (https://lamia.fi)
 */


class Verifone_VerifonePayment_Block_Adminhtml_Summary extends Mage_Adminhtml_Block_System_Config_Form_Field
{
    protected function _prepareLayout()
    {
        parent::_prepareLayout();
        if (!$this->getTemplate()) {
            $this->setTemplate('VerifonePayment/configuration_summary.phtml');
        }
        return $this;
    }

    public function render(Varien_Data_Form_Element_Abstract $element)
    {
        $element->unsScope()->unsCanUseWebsiteValue()->unsCanUseDefaultValue();
        return parent::render($element);
    }

    public function getWebsiteName()
    {
        $websiteName = Mage::helper('core')->__('Default');

        if (!empty($code = Mage::app()->getRequest()->getParam('website'))) // website level
        {
            $website = Mage::getModel('core/website')->load($code);
            $websiteName = $website->getName();
        }

        return $websiteName;
    }

    public function getConfigurationDataForDisplay()
    {

        $helper = Mage::helper('VerifonePayment/configuration');

        if (!empty($code = Mage::app()->getRequest()->getParam('website'))) // website level
        {
            $websiteId = Mage::getModel('core/website')->load($code)->getId();
            $storeId = Mage::app()->getWebsite($websiteId)->getDefaultStore()->getId();
        } else // default level
        {
            $storeId = 0;
        }

        /** Data for display */
        $display = array();

        $display['isLiveMode'] = array(
            'label' => $helper->__('Mode'),
            'value' => Mage::getStoreConfig('payment/VerifonePayment/is_live_mode', $storeId) ? $helper->__('Production') : $helper->__('Test'),
            'has_desc' => false,
            'has_desc_class' => false
        );

        $display['merchantCode'] = array(
            'label' => $helper->__('Verifone Payment merchant agreement code'),
            'value' => $helper->getMerchantAgreement($storeId),
            'has_desc' => false,
            'has_desc_class' => false
        );
        if ($helper->getMerchantAgreement($storeId) === $helper->getMerchantAgreementDefault($storeId)) {
            $display['merchantCode']['desc'] = $helper->__('Default test merchant agreement uses');
            $display['merchantCode']['desc_class'] = 'info';
            $display['merchantCode']['has_desc'] = true;
            $display['merchantCode']['has_desc_class'] = true;
        }

        $display['delayedUrl'] = array(
            'label' => $helper->__('Delayed success url'),
            'value' => Mage::getUrl('VerifonePayment/redirect/successDelayed'),
            'desc' => $helper->__('This is the url that you need to copy to payment provider settings in their portal.'),
            'desc_class' => 'success',
            'has_desc' => true,
            'has_desc_class' => true
        );

        $display['keyHandlingMode'] = array(
            'label' => $helper->__('Key handling mode'),
            'value' => $helper->isKeySimpleMode($storeId) ? $helper->__('Automatic (Simple)') : $helper->__('Manual (Advanced)'),
            'has_desc' => false,
            'has_desc_class' => false
        );

        $display['paymentServiceKey'] = array(
            'label' => $helper->__('Path and filename of Verifone Payment public key file'),
            'value' => $helper->getPaymentPublicKeyPath($storeId),
            'has_desc' => false,
            'has_desc_class' => false
        );

        if (file_exists($helper->getPaymentPublicKeyPath($storeId))) {
            $display['paymentServiceKey']['desc'] = $helper->__('Key file is available');
            $display['paymentServiceKey']['desc_class'] = 'success';
            $display['paymentServiceKey']['has_desc'] = true;
            $display['paymentServiceKey']['has_desc_class'] = true;
        } else {
            $display['paymentServiceKey']['desc'] = $helper->__('Problem with load key file. Please contact with customer service');
            $display['paymentServiceKey']['desc_class'] = 'success';
            $display['paymentServiceKey']['has_desc'] = true;
            $display['paymentServiceKey']['has_desc_class'] = true;
        }

        if ($helper->isKeyAdvancedMode($storeId)) {

            $path = $helper->getKeysDirectory($storeId);

            $display['directory'] = array(
                'label' => $helper->__('Directory for store keys'),
                'value' => $path,
                'has_desc' => false,
                'has_desc_class' => false
            );
            if (file_exists($path) && is_writable($path)) {
                $display['directory']['desc'] = $helper->__('Directory configured properly');
                $display['directory']['desc_class'] = 'success';
                $display['directory']['has_desc'] = true;
                $display['directory']['has_desc_class'] = true;
            } else {
                $display['directory']['desc'] = $helper->__('Problem with directory configuration. Please check configuration and save.');
                $display['directory']['desc_class'] = 'error';
                $display['directory']['has_desc'] = true;
                $display['directory']['has_desc_class'] = true;
            }

            if($helper->getMerchantAgreementDefault($storeId) !== $helper->getMerchantAgreement($storeId)) {
                $display['shopPrivateKey'] = array(
                    'label' => $helper->__('Path and filename of shop private key file'),
                    'value' => $helper->getShopPrivateKeyPath($storeId),
                    'has_desc' => false,
                    'has_desc_class' => false
                );

                if (file_exists($display['shopPrivateKey']['value']) && !empty($helper->getShopPrivateKeyFileName($storeId))) {
                    $display['shopPrivateKey']['desc'] = $helper->__('Key file is available');
                    $display['shopPrivateKey']['desc_class'] = 'success';
                    $display['shopPrivateKey']['has_desc'] = true;
                    $display['shopPrivateKey']['has_desc_class'] = true;
                } else {
                    $display['shopPrivateKey']['desc'] = $helper->__('Key file is not available');
                    $display['shopPrivateKey']['desc_class'] = 'error';
                    $display['shopPrivateKey']['has_desc'] = true;
                    $display['shopPrivateKey']['has_desc_class'] = true;
                }
            } else {
                $display['shopPrivateKey'] = array(
                    'label' => $helper->__('Path and filename of shop private key file'),
                    'value' => '',
                    'has_desc' => true,
                    'has_desc_class' => true,
                    'desc' => $helper->__('Default key file is used'),
                    'desc_class' => 'info'
                );
            }
        } else {

            $display['shopPrivateKey'] = array(
                'label' => $helper->__('Path and filename of shop private key file'),
                'value' => 'Key file stored in database',
                'has_desc' => false,
                'has_desc_class' => false
            );

            if ($helper->getShopPrivateKey($storeId) !== null && $helper->getShopPrivateKey($storeId) !== $helper->getShopPrivateKeyDefault($storeId)) {
                $display['shopPrivateKey']['desc'] = $helper->__('Key file is available');
                $display['shopPrivateKey']['desc_class'] = 'success';
                $display['shopPrivateKey']['has_desc'] = true;
                $display['shopPrivateKey']['has_desc_class'] = true;
            } elseif(!$helper->isLiveMode($storeId) && $helper->getMerchantAgreement($storeId) === $helper->getMerchantAgreementDefault($storeId)) {
                $display['shopPrivateKey']['desc'] = $helper->__('Default key file is used');
                $display['shopPrivateKey']['desc_class'] = 'info';
                $display['shopPrivateKey']['has_desc'] = true;
                $display['shopPrivateKey']['has_desc_class'] = true;
            } else {
                $display['shopPrivateKey']['desc'] = $helper->__('Problem with fetch shop private key file. Please check configuration and/or generate key');
                $display['shopPrivateKey']['desc_class'] = 'error';
                $display['shopPrivateKey']['has_desc'] = true;
                $display['shopPrivateKey']['has_desc_class'] = true;
            }
        }

        if ($helper->isKeySimpleMode($storeId) && $helper->getShopPublicKey($storeId) === null && $helper->getMerchantAgreement($storeId) !== $helper->getMerchantAgreementDefault($storeId)) {
            $display['shopPublicKeyContent'] = array(
                'label' => $helper->__('Public key file'),
                'value' => '',
                'has_desc' => true,
                'has_desc_class' => true,
                'desc' => $helper->__('Problem with fetch shop public key file. Please check configuration and/or generate key'),
                'desc_class' => 'error'
            );
        } elseif ($helper->getShopPublicKey($storeId) !== null) {
            $display['shopPublicKeyContent'] = array(
                'label' => $helper->__('Public key file'),
                'value' => $helper->getShopPublicKey($storeId),
                'has_desc' => true,
                'has_desc_class' => true,
                'desc' => $helper->__('Please, copy this key to payment operator configuration settings, otherwise, the payment will be broken.'),
                'desc_class' => 'success'
            );
        }

        return $display;
    }

    protected function _getElementHtml(Varien_Data_Form_Element_Abstract $element)
    {
        $originalData = $element->getOriginalData();
        $this->addData(array(
            'button_label' => Mage::helper('customer')->__($originalData['button_label']),
            'html_id' => $element->getHtmlId(),
            'ajax_url' => Mage::getSingleton('adminhtml/url')->getUrl('*/verifone_generateKeys')
        ));

        return $this->_toHtml();
    }
}