<?php

/*
 */

class Verifone_VerifonePayment_Block_Payment_Methods_Form extends Mage_Payment_Block_Form
{
    public function _construct()
    {
        parent::_construct();

        if(!Mage::app()->getStore()->isAdmin()) {
            $this->setTemplate('VerifonePayment/form.phtml');
        }
    }

    public function getCodeInput()
    {
        $methodCode = $this->getGateCode();

        $html = '<input type="hidden" name="payment[s-t-1-30_payment-method-code]" value="' . $methodCode . '">';

        return $html;
    }

    public function getGateCode()
    {
        return $this->getMethod()->getGateCode();
    }

    public function getGroupsSerializedData($_asJson = true)
    {
        $_cardsPayments = unserialize(Mage::getStoreConfig('payment/VerifonePayment/cardpaymentsgroup_array'));
        $_customGroupsPayments = unserialize(Mage::getStoreConfig('payment/VerifonePayment/paymentsgroups_array'));

        $groupedPayments = array_merge($_customGroupsPayments, $_cardsPayments);

        if (!$_asJson) {
            return $groupedPayments;
        }
        return str_replace(array('\r\n', '\n'),'<br \/>', json_encode($groupedPayments));
    }

    public function isCard()
    {
        return $this->getMethod()->isCard();
    }

    public function isRecurring()
    {
        return is_a($this->getMethod(), 'Verifone_VerifonePayment_Model_Payment_Recurring');
    }

    public function isLoggedIn()
    {
        return Mage::getSingleton('customer/session')->isLoggedIn();
    }

    public function allowToRemember()
    {
        return $this->isCard()
        && $this->isLoggedIn()
        && Mage::getStoreConfig('payment/VerifonePayment/allow_to_save_cc');
    }

    public function shouldAddPrefix()
    {
        return count($this->getSavedPayments()) && $this->allowToRemember();
    }

    protected function getCardOptions()
    {
        $cardOptions = array();
        $cardCodes = array();
        $allCardOptions = Mage::getModel('VerifonePayment/db_pay_options')->getCardsToOptionArray();
        $cardGroups = unserialize(Mage::getStoreConfig('payment/VerifonePayment/cardpaymentsgroup_array'));

        foreach ($cardGroups as $cardGroup) {
            if (isset($cardGroup['payments']) && is_array($cardGroup['payments'])) {
                $cardCodes = array_merge($cardCodes, array_values($cardGroup['payments']));
            }
        }

        foreach ($allCardOptions as $cardOption) {
            if (isset($cardOption['value']) && in_array($cardOption['value'], $cardCodes)) {
                $cardOptions[] = $cardOption;
            }
        }

        return $cardOptions;
    }

    protected function getSavedPayments()
    {
        return Mage::getModel('VerifonePayment/db_pay_saved')->getSavedPayments();
    }

    protected function getSavedPaymentsAsJson()
    {
        $savedPayments = $this->getSavedPayments();
        $cardOptions = $this->getCardOptions();

        /** @var $method Verifone_VerifonePayment_Model_Payment */
        foreach ($cardOptions as $cardOption) {
            $code = $cardOption['value'];
            $name = $cardOption['label'];
            $savedPayments[$code] = array(
                'l-t-1-20_payment-method-id' => $code,
                's-t-1-30_payment-method-code' => $code,
                's-t-1-30_payment-method-title' => $this->helper('VerifonePayment')->__('Add new ' . $name),
                'is_default' => false
            );
        }

        return !empty($savedPayments) ? json_encode($savedPayments) : '';
    }

    protected function getGroupMessage()
    {
        $groupedPayments = $this->getGroupsSerializedData(false);

        foreach ($groupedPayments as $group) {

            if (in_array($this->getGateCode(), $group['payments'])) {
                if (isset($group['group_description']) && $group['group_description']) {
                    return $group['group_description'];
                }

                return $this->getDefaultMessage();
            }
        }

        return $this->getDefaultMessage();
    }

    protected function getDefaultMessage()
    {
        return $this->__('You will be redirected to Verifone Payment service to complete your order.');
    }

}
