<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is released under commercial license by Lamia Oy.
 *
 * @copyright Copyright (c) 2018 Lamia Oy (https://lamia.fi)
 */


class Verifone_VerifonePayment_Helper_Configuration extends Mage_Core_Helper_Abstract
{
    public function getTestLiveConfig($path)
    {
        if (Mage::getStoreConfig('payment/VerifonePayment/is_live_mode')) {
            return Mage::getStoreConfig('payment/VerifonePayment/' . $path);
        }

        return Mage::getStoreConfig('payment/VerifonePayment/' . $path . '_test');
    }

    public function getMerchantAgreement($storeId = null)
    {

        if (Mage::getStoreConfig('payment/VerifonePayment/is_live_mode', $storeId)) {
            return Mage::getStoreConfig('payment/VerifonePayment/merchant_agreement_code', $storeId);
        }

        if (!empty(Mage::getStoreConfig('payment/VerifonePayment/merchant_agreement_code_test', $storeId))) {
            return Mage::getStoreConfig('payment/VerifonePayment/merchant_agreement_code_test', $storeId);
        }

        return Mage::getStoreConfig('payment/VerifonePayment/merchant_agreement_code_default', $storeId);

    }

    public function getMerchantAgreementDefault($storeId = null)
    {
        return Mage::getStoreConfig('payment/VerifonePayment/merchant_agreement_code_default', $storeId);
    }

    public function getKeyMode($storeId = null)
    {
        return Mage::getStoreConfig('payment/VerifonePayment/key_handling_mode', $storeId);
    }

    public function isKeySimpleMode($storeId = null)
    {
        return (int)$this->getKeyMode($storeId) == Verifone_VerifonePayment_Model_Adminhtml_System_Config_Source_HandlingMode::SIMPLE_MODE;
    }

    public function isKeyAdvancedMode($storeId = null)
    {
        return (int)$this->getKeyMode($storeId) == Verifone_VerifonePayment_Model_Adminhtml_System_Config_Source_HandlingMode::ADVANCED_MODE;
    }

    /** KEYS FILES */
    public function isLiveMode($storeId = null)
    {
        return Mage::getStoreConfig('payment/VerifonePayment/is_live_mode', $storeId);
    }

    public function getKeysDirectory($storeId = null)
    {
        return Mage::getStoreConfig('payment/VerifonePayment/keyfile_directory', $storeId);
    }

    public function getShopPrivateKeyFileName($storeId = null)
    {
        if ($this->isLiveMode($storeId)) {
            return Mage::getStoreConfig('payment/VerifonePayment/shop_private_keyfile', $storeId);
        }

        return Mage::getStoreConfig('payment/VerifonePayment/shop_private_keyfile_test', $storeId);
    }

    public function getLiveShopPrivateKeyPath($storeId = null)
    {

        if ($this->isKeySimpleMode($storeId)) {
            return null;
        }

        return $this->getKeysDirectory($storeId) . DIRECTORY_SEPARATOR . Mage::getStoreConfig('payment/VerifonePayment/shop_private_keyfile', $storeId);

    }

    public function getTestShopPrivateKeyPath($storeId = null)
    {

        if ($this->isKeySimpleMode($storeId)) {
            return null;
        }

        return $this->getKeysDirectory($storeId) . DIRECTORY_SEPARATOR . Mage::getStoreConfig('payment/VerifonePayment/shop_private_keyfile_test', $storeId);
    }

    public function getShopPrivateKeyPath($storeId = null)
    {
        if ($this->isLiveMode($storeId)) {
            return $this->getLiveShopPrivateKeyPath($storeId);
        }

        return $this->getTestShopPrivateKeyPath($storeId);
    }

    public function getShopPrivateKeyFile($storeId = null)
    {
        return $this->getShopPrivateKey($storeId);
    }

    public function getShopPrivateKey($storeId = null)
    {

        // If TEST mode is set
        if (!$this->isLiveMode($storeId)) {

            if ($this->getMerchantAgreement($storeId) === $this->getMerchantAgreementDefault($storeId)) {
                // If DEFAULT test merchant is set, return default key
                return $this->getShopPrivateKeyDefault($storeId);
            }

            if ($this->isKeySimpleMode($storeId)) {
                // If CUSTOM test merchant is set, and SIMPLE mode is set, return generated key stored in DB
                return Mage::helper('VerifonePayment/keys')->getTestPrivateKey($storeId);
            }

            $path = $this->getTestShopPrivateKeyPath($storeId);
            if (file_exists($path)) {
                // If CUSTOM test merchant is set, and ADVANCED mode is set, return old key stored in files
                return file_get_contents($path);
            }

            // return default key file
            return $this->getShopPrivateKeyDefault($storeId);
        }

        // If LIVE mode is set

        if ($this->isKeySimpleMode($storeId)) {
            // If CUSTOM test merchant is set, and SIMPLE mode is set, return generated key stored in DB
            return Mage::helper('VerifonePayment/keys')->getLivePrivateKey($storeId);
        }

        $path = $this->getLiveShopPrivateKeyPath($storeId);
        if (file_exists($path)) {
            // If CUSTOM test merchant is set, and ADVANCED mode is set, return old key stored in files
            return file_get_contents($path);
        }

        // return nothing
        return null;
    }

    public function getShopPrivateKeyPathDefault($storeId = null)
    {
        return Mage::getModuleDir('', 'Verifone_VerifonePayment') . DIRECTORY_SEPARATOR . 'keys' . DIRECTORY_SEPARATOR . Mage::getStoreConfig('payment/VerifonePayment/shop_private_keyfile_default', $storeId);
    }

    public function getShopPrivateKeyDefault($storeId = null)
    {
        return file_get_contents($this->getShopPrivateKeyPathDefault($storeId));
    }

    public function getShopPublicKey($storeId = null)
    {

        // If TEST mode is set
        if (!$this->isLiveMode($storeId)) {

            if ($this->getMerchantAgreement($storeId) === $this->getMerchantAgreementDefault($storeId)) {
                // If DEFAULT test merchant is set, return default key
                // For default simple key is not require to configure in payment service.
                return null;
            }

            if ($this->isKeySimpleMode($storeId)) {
                // If CUSTOM test merchant is set, and SIMPLE mode is set, return generated key stored in DB
                return Mage::helper('VerifonePayment/keys')->getTestPublicKey($storeId);
            }


            // If CUSTOM test merchant is set, and ADVANCED mode is set, return old key stored in files
            // This is not required, because if this mode is set, it means that payment service is configured,
            // and it does not require to configure again.
            return null;
        }

        // If LIVE mode is set
        if ($this->isKeySimpleMode($storeId)) {
            // If CUSTOM test merchant is set, and SIMPLE mode is set, return generated key stored in DB
            return Mage::helper('VerifonePayment/keys')->getLivePublicKey($storeId);
        }

        // When advanced mode is set, or is default merchant agreement then in not require simple key to display.
        return null;
    }

    public function getShopPublicKeyPathDefault($storeId = null)
    {
        return Mage::getModuleDir('', 'Verifone_VerifonePayment') . DIRECTORY_SEPARATOR . 'keys' . DIRECTORY_SEPARATOR . Mage::getStoreConfig('payment/VerifonePayment/shop_public_keyfile_default', $storeId);
    }

    public function getShopPublicKeyDefault($storeId = null)
    {
        return file_get_contents($this->getShopPublicKeyPathDefault($storeId));
    }

    public function getPaymentPublicKeyPath($storeId = null)
    {
        if ($this->isLiveMode($storeId)) {
            return Mage::getModuleDir('', 'Verifone_VerifonePayment') . DIRECTORY_SEPARATOR . 'keys' . DIRECTORY_SEPARATOR . Mage::getStoreConfig('payment/VerifonePayment/pay_page_public_keyfile_live_default', $storeId);
        }

        return Mage::getModuleDir('', 'Verifone_VerifonePayment') . DIRECTORY_SEPARATOR . 'keys' . DIRECTORY_SEPARATOR . Mage::getStoreConfig('payment/VerifonePayment/pay_page_public_keyfile_test_default', $storeId);
    }

    public function getPaymentPublicKeyFile($storeId = null)
    {
        return file_get_contents($this->getPaymentPublicKeyPath($storeId));
    }

}