<?php

/*
 * Cryptutils functions for VerifonePayment
 */

class Verifone_VerifonePayment_Helper_Cryptutils extends Mage_Core_Helper_Abstract
{
    public function __construct()
    {
        if(Mage::getStoreConfig('payment/VerifonePayment/disable_rsa_blinding')) {
            define('CRYPT_RSA_DISABLE_BLINDING', true);
        }
    }

    // Cryptutils functions
  public function formatparameters($parameters)
  {
    ksort($parameters, SORT_STRING);
    $formatedresult = "";
    foreach ($parameters as $key => $value) {
      $value = str_replace(';', ';;', $value);
      $formatedresult .= sprintf("%s=%s;", $key, $value);
    }
    return $formatedresult;
  }

  /** Creates signature for data
   * @param $data string data to create signature
   * @param $storeId string
   * @return string
   */
  public function generatesignature($data, $storeId = null)
  {
    set_include_path(get_include_path() . PATH_SEPARATOR . dirname(__FILE__));
    $key = Mage::helper('VerifonePayment/configuration')->getShopPrivateKeyFile($storeId);
    if ($key === false) {
      $this->raise_error($this->__('Key file could not be read.'));
      return false;
    }
    require_once('Crypt/RSA.php');
    $rsa = new Crypt_RSA();
    $rsa->setSignatureMode(CRYPT_RSA_SIGNATURE_PKCS1);
    $rsa->loadKey($key);
    $signature = $rsa->sign($data);
    return strtoupper(bin2hex($signature));
  }

  /** Verifies the data signature
   * @param $datatoverify string - data to verify signature for
   * @param $signaturedata string - signature data as hex string
   * @param $storeId string
   * @return bool true - signature valid, false - invalid
   */
  public function verifysignature($datatoverify, $signaturedata, $storeId = null)
  {
    set_include_path(get_include_path() . PATH_SEPARATOR . dirname(__FILE__));
    $key = Mage::helper('VerifonePayment/configuration')->getPaymentPublicKeyFile($storeId);
    if ($key === false) {
      $this->raise_error($this->__('Key file could not be read.'));
      return false;
    }

    require_once('Crypt/RSA.php');
    $rsa = new Crypt_RSA();
    $rsa->setSignatureMode(CRYPT_RSA_SIGNATURE_PKCS1);
    $rsa->loadKey($key);
    return $rsa->verify($datatoverify, pack("H*", $signaturedata)) ? true : false;
  }

  protected function raise_error($msg)
  {
    Mage::throwException($msg);
  }

}
