<?php

class Verifone_VerifonePayment_Model_Api_Paymentinterface_Abstract extends Mage_Payment_Model_Method_Abstract
{
    protected $_orderNote;
    protected $_fields;

    public function __construct()
    {
        parent::__construct();
        $this->_construct();
    }

    public function _construct()
    {
        parent::_construct();
        $this->prepareData();
    }

    // Return payment url to be used for connecting to the payment system
    // This function implements the Availability Get operation in the Verifone-e-commerce-payment interface
    // Required configuration: allow_url_fopen must be enabled
    public function getUrl()
    {
        if (!Mage::getStoreConfig('payment/VerifonePayment/is_live_mode')) {
            return $payment_urls[0] = Mage::getStoreConfig('payment/VerifonePayment/pay_page_url_demo');
        }

        // check that the configuration is correct
        if (!(ini_get('allow_url_fopen'))) {
            Mage::throwException("Configuration error: allow_url_fopen must be enabled");
        }

        $payment_urls = array();
        $payment_urls[] = Mage::getStoreConfig('payment/VerifonePayment/pay_page_url_one');
        $second_url = Mage::getStoreConfig('payment/VerifonePayment/pay_page_url_two');
        $third_url = Mage::getStoreConfig('payment/VerifonePayment/pay_page_url_three');

        if (!empty($second_url)) {
            $payment_urls[] = $second_url;
        }
        if (!empty($third_url)) {
            $payment_urls[] = $third_url;
        }

        foreach ($payment_urls as $url) {
            if (!Mage::getStoreConfig('payment/VerifonePayment/validate_url')) {
                return $url;
            }
            // GET must result HTTP 200 OK and empty contents
            // anything else indicates that the payment system node is not available
            if ($this->isUrlAvailable($url)) {
                return $url;
            }
        }
        // flag error
        return false;
    }

    public function getFormFields()
    {
        $_fields = $this->getPreparedFields();

        Mage::dispatchEvent('verifone_paymentinterface_send_request_before', array(
            '_class' => get_class($this),
            '_fields' => $_fields
        ));

        return $_fields;
    }

    public function getPreparedFields()
    {
        if (is_null($this->_fields)) {
            return $this->prepareData();
        }
        return $this->_fields;
    }

    public function prepareData()
    {
        if (is_null($this->_fields)) {
            $this->_fields = array();
        }

        $this->_fields = array_merge($this->_getUrlsPIForm(), $this->_fields);

        $this->_fields = array_filter($this->_fields, 'strlen');

        if (!Mage::getStoreConfig('is_local')) {
            $this->_fields = $this->getSignatureOne($this->_fields);
        }

        return $this->_fields;
    }

    protected function _getUrlsPIForm()
    {
        return Mage::helper('VerifonePayment/url')->getDefaultReturnUrls();

    }

    protected function getSignatureOne($fields)
    {

        $cryptutils = Mage::helper('VerifonePayment/cryptutils');
        $fields["s-t-256-256_signature-one"] = $cryptutils->generatesignature($cryptutils->formatparameters($fields));

        return $fields;
    }

    public function setOrderNote($_orderNote)
    {
        $this->_orderNote = $_orderNote;
        return $this;
    }

    protected function getOrderNote()
    {
        if (is_null($this->_orderNote)) {
            $this->setOrderNote(time());
        }
        return $this->_orderNote;
    }

    /**
     * @param string $url
     *
     * @type bool
     */
    protected function isUrlAvailable($url)
    {
        $checker = \Mage::getSingleton('VerifonePayment/urlStatusChecker');
        $checker->requestUrlStatus($url);

        return $checker->getResponseStatusCode() >= 200
            && $checker->getResponseStatusCode() < 300
            && !$checker->getResponseBody();
    }
}
