<?php

class Verifone_VerifonePayment_Model_Api_Paymentinterface_PaymentInitialization extends Verifone_VerifonePayment_Model_Api_Paymentinterface_Abstract
{
    const BASKET_LIMIT = 48;

    protected $_paymentMethodCode;
    protected $_isRecurring;

    protected $_customer;
    protected $_quote;
    protected $_order;
    protected $_profile;


    public function _construct()
    {
        parent::_construct();
    }

    protected function getPaymentMethodCode()
    {
        if (!$this->getOrder()->getPayment()) {
            return '';
        }
        $additionalInformation = $this->getOrder()->getPayment()->getAdditionalInformation();
        return isset($additionalInformation['s-t-1-30_payment-method-code']) && $additionalInformation['s-t-1-30_payment-method-code'] != 'VerifonePayment' ? $additionalInformation['s-t-1-30_payment-method-code'] : '';
    }

    protected function getSavePaymentMethod()
    {
        if (!$this->getOrder()->getPayment()) {
            return '';
        }
        $additionalInformation = $this->getOrder()->getPayment()->getAdditionalInformation();
        return isset($additionalInformation['i-t-1-1_save-payment-method']) && $additionalInformation['i-t-1-1_save-payment-method'] ? '1' : '3';
    }

    protected function getSavedPaymentMethodId()
    {
        if (!$this->getOrder()->getPayment()) {
            return '';
        }
        $additionalInformation = $this->getOrder()->getPayment()->getAdditionalInformation();
        return isset($additionalInformation['l-t-1-20_saved-payment-method-id']) && $additionalInformation['l-t-1-20_saved-payment-method-id'] ? $additionalInformation['l-t-1-20_saved-payment-method-id'] : '';
    }

    protected function getOrderGrossAmount()
    {
        return round($this->getOrder()->getGrandTotal(), 2) * 100;
    }

    protected function getOrderNetAmount()
    {
        return $this->getOrderGrossAmount() - $this->getOrderVatAmount();
    }

    protected function getOrderVatAmount()
    {
        return round($this->getOrder()->getTaxAmount(), 2) * 100;
    }

    public function setQuote($_quote)
    {
        $this->_quote = $_quote;

        return $this;
    }

    protected function getQuote()
    {
        if (!$this->_quote) {
            $_session = Mage::getSingleton('checkout/session');
            $_quote = $_session->getQuote();
            $this->setQuote($_quote);
        }

        return $this->_quote;
    }

    protected function setIsRecurring($_bool = true)
    {
        $this->_isRecurring = $_bool;

        return $this;
    }

    protected function isRecurring()
    {
        if (is_null($this->_isRecurring)) {
            $this->setIsRecurring(false);
        }

        return $this->_isRecurring;
    }

    public function setProfile($_profile)
    {
        $this->_profile = $_profile;

        if ($_profile) {
            $this->setIsRecurring();

            $_order = Mage::getResourceModel('sales/order_collection')
                ->addRecurringProfilesFilter($_profile->getId())
                ->setOrder('entity_id', 'desc')
                ->getFirstItem();

            $this->setOrder($_order);
        }

        return $this;
    }

    public function getProfile()
    {
        return $this->_profile;
    }

    public function setOrder($_order)
    {
        $this->_order = $_order;

        return $this;
    }

    /**
     * @return Mage_Sales_Model_Order
     */
    protected function getOrder()
    {
        if (!$this->_order) {
            $_quote = $this->getQuote();
            $_orderIncrementId = $_quote ? $_quote->getReservedOrderId() : null;

            if (is_null($_orderIncrementId)) {
                $payment = Mage::getModel('VerifonePayment/payment_recurring');
                $profileIds = $payment->getCheckout()->getLastRecurringProfileIds();

                if (is_array($profileIds)) {
                    foreach ($profileIds as $profileId) {
                        $profile = Mage::getModel('sales/recurring_profile')->load($profileId);

                        if ($this->setProfile($profile)->getProfile()) {
                            return $this->_order;
                        }
                    }
                }
            }

            $_order = Mage::getModel('sales/order')->loadByIncrementId($_orderIncrementId);
            $this->setOrder($_order);
        }

        return $this->_order;
    }

    public function prepareData()
    {
        $_helper = Mage::helper('VerifonePayment');

        $_order = $this->getOrder();
        $_requestTimestamp = gmdate('Y-m-d H:i:s');
        $_token = sprintf("%s;%s;%s", Mage::getStoreConfig('payment/VerifonePayment/merchant_agreement_code'), $_order->getIncrementId(), $_requestTimestamp);

        $_fields = array(
            's-f-32-32_payment-token' => strtoupper(substr(hash('sha256', $_token), 0, 32)),
            'locale-f-2-5_payment-locale' => Mage::app()->getLocale()->getLocaleCode(),
            't-f-14-19_payment-timestamp' => $_requestTimestamp,
            's-f-1-36_merchant-agreement-code' => Mage::getStoreConfig('payment/VerifonePayment/merchant_agreement_code'),
            's-f-1-30_software' => 'Magento',
            's-f-1-10_software-version' => $_helper->_sanitise(Mage::getVersion(), 10, true),
            'i-f-1-11_interface-version' => '5',

            's-f-1-36_order-number' => $_order->getIncrementId(),
            't-f-14-19_order-timestamp' => $_order->getCreatedAt(),
            //'s-t-1-36_order-note' => $this->getOrderNote(),                                                   // not obligatory
            'i-f-1-3_order-currency-code' => $_helper->convertCountryToISO4217($_order->getOrderCurrency()->getCurrencyCode()),
            'l-f-1-20_order-gross-amount' => $this->getOrderGrossAmount(),
            'l-f-1-20_order-net-amount' => $this->getOrderNetAmount(),
            'l-f-1-20_order-vat-amount' => $this->getOrderVatAmount(),
            //'i-t-1-4_order-vat-percentage' => '',                                                             // not obligatory
            's-t-1-30_payment-method-code' => $this->getPaymentMethodCode(),
            'i-t-1-1_skip-confirmation-page' => '1',
            'i-t-1-1_save-payment-method' => $this->getSavePaymentMethod(),
            'l-t-1-20_saved-payment-method-id' => $this->getSavedPaymentMethodId(),

            's-f-1-30_buyer-first-name' => $this->getFirstName($_order),
            's-f-1-30_buyer-last-name' => $this->getLastname($_order),
            's-t-1-30_buyer-phone-number' => $this->getPhoneNumber(),
            's-f-1-100_buyer-email-address' => $_helper->_sanitise($_order->getBillingAddress()->getEmail(), 100),
        );

        if(Mage::getStoreConfig('payment/VerifonePayment/save_masked_pan_number')) {
            $_fields['s-t-1-1024_dynamic-feedback'] = 'i-t-2-2_card-pan-last2,i-t-6-6_card-pan-first6';
        }

        //Downloadable products don't have shipping address
        if ($_order->getShippingAddress() !== false) {
            $_fields['s-t-1-30_delivery-address-line-one'] = $_helper->_sanitise($_order->getShippingAddress()->getStreet1(), 30);
            $_fields['s-t-1-30_delivery-address-line-two'] = $_helper->_sanitise($_order->getShippingAddress()->getStreet2(), 30) ?
                $_helper->_sanitise($_order->getShippingAddress()->getStreet2(), 30) : $_helper->_sanitise($_order->getShippingAddress()->getRegion(), 30);
            $_fields['s-t-1-30_delivery-address-line-three'] = $_helper->_sanitise($_order->getShippingAddress()->getStreet2(),
                30) ? $_helper->_sanitise($_order->getShippingAddress()->getRegion(), 30) : '';
            $_fields['s-t-1-30_delivery-address-city'] = $_helper->_sanitise($_order->getShippingAddress()->getCity(), 30);
            $_fields['s-t-1-30_delivery-address-postal-code'] = $_helper->_sanitise($_order->getShippingAddress()->getPostcode(), 30);
            $_fields['i-t-1-3_delivery-address-country-code'] = $_helper->convertCountryCode2Numeric($_order->getShippingAddress()->getCountry());
        }

        if ($this->isRecurring()) {

            $_billingAddress = $this->getProfile()->getBillingAddressInfo();

            $_fields['i-t-1-1_recurring-payment'] = '1';
            //$_fields['i-t-1-1_deferred-payment'] = '0';
            $_fields['s-t-1-30_recurring-payment-subscription-name'] = strtoupper(substr($_billingAddress['firstname'] . ' ' . $_billingAddress['lastname'], 0, 30));
            $_fields['s-t-1-30_recurring-payment-subscription-code'] = $this->getProfile()->getReferenceId();
            $_fields['i-t-1-3_recurring-payment-subscription-expected-period'] = '7';
            $_fields['t-t-14-19_recurring-payment-subscription-end]'] = '';
            $_fields['i-t-1-1_save-payment-method'] = '1';
        }

        if ($this->isSendBasketItems()) {
            $_fields = array_merge(
                $_fields,
                $this->getBasketItemsData()
            );
        }

        $this->_fields = $_fields;

        return parent::prepareData();
    }

    protected function getFirstName($_order)
    {
        if (!$_order->getBillingAddress()) {
            return '';
        }

        return substr($_order->getBillingAddress()->getFirstname(), 0, 30);
    }

    protected function getLastname($_order)
    {
        if (!$_order->getBillingAddress()) {
            return '';
        }

        return substr($_order->getBillingAddress()->getLastname(), 0, 30);
    }

    protected function getBasketItemsData()
    {
        $_fields = array();

        if (!$this->isSendBasketItems()) {
            return $_fields;
        }

        $i = $items_tax = $items_net_price = $items_gross_price = 0;
        $items = $this->getOrder()->getAllVisibleItems();

        if (count($items) >= self::BASKET_LIMIT) {
            $items = $this->groupItemsByVat($items);
        }

        /** @var Mage_Sales_Model_Order_Item $_item */
        foreach ($items as $_item) {
            $item_count = $_item->getQtyToInvoice() ? (int)$_item->getQtyToInvoice() : (int)$_item->getQtyOrdered();

            $_basketItemFields = $this->getBasketItemData($_item, $i, $item_count);

            Mage::dispatchEvent(
                'verifone_prepare_basket_item_after',
                array(
                    'storeId' => $this->getOrder()->getStoreId(),
                    'item' => $_item,
                    'basketItem' => &$_basketItemFields,
                    'count' => $i
                )
            );

            if (!$this->isCombineBasketItems()) {
                $_fields = array_merge(
                    $_fields,
                    $_basketItemFields
                );
            }

            $items_tax += $_basketItemFields["l-t-1-20_bi-gross-amount-$i"] - $_basketItemFields["l-t-1-20_bi-net-amount-$i"];
            $items_net_price += $_basketItemFields["l-t-1-20_bi-net-amount-$i"];
            $items_gross_price += $_basketItemFields["l-t-1-20_bi-gross-amount-$i"];

            $i++;
        }

        if ($this->getOrder()->getShippingAmount() != 0) {
            $shippingFields = $this->getBasketShippingAmountData($i);

            if (!$this->isCombineBasketItems()) {
                $_fields = array_merge(
                    $_fields,
                    $shippingFields
                );
            }

            $items_tax += $shippingFields["l-t-1-20_bi-gross-amount-$i"] - $shippingFields["l-t-1-20_bi-net-amount-$i"];
            $items_net_price += $shippingFields["l-t-1-20_bi-net-amount-$i"];
            $items_gross_price += $shippingFields["l-t-1-20_bi-gross-amount-$i"];
            $i++;
        }

        if (!$this->isCombineBasketItems()) {
            $discountAmount = $this->getOrderGrossAmount() - $items_gross_price;
            if (abs($discountAmount) >= 1) {
                $_fields = array_merge(
                    $_fields,
                    $this->getBasketDiscountData($i++, $this->getOrderGrossAmount(), $items_gross_price,
                        $this->getOrderNetAmount(), $items_net_price)
                );
            }
        }

        if ($this->isCombineBasketItems()) {
            $i = 0; // Reset items index, since we will only have 1 combined item.
            $_fields = array_merge(
                $_fields,
                $this->getCombinedBasketItemsData($i, $items_gross_price, $items_net_price)
            );
        }

        return $_fields;
    }

    /**
     * @param Mage_Sales_Model_Order_Item[] $items
     * @return Mage_Sales_Model_Order_Item[]
     */
    protected function groupItemsByVat($items)
    {
        /** @var Mage_Sales_Model_Order_Item[] $result */
        $result = array();

        foreach ($items as $item) {
            $tax = (string)$item->getTaxPercent();

            if (!isset($result[$tax])) {
                $mergedItem = new Mage_Sales_Model_Order_Item();
                $mergedItem->setName('Multiple items - vat ' . $tax);
                $mergedItem->setPrice(0);
                $mergedItem->setPriceInclTax(0);
                $mergedItem->setTaxPercent($item->getTaxPercent());
                $mergedItem->setQtyOrdered(1);

                $result[$tax] = $mergedItem;
            }

            $result[$tax]->setPrice($result[$tax]->getPrice() + $item->getRowTotal());
            $result[$tax]->setPriceInclTax($result[$tax]->getPriceInclTax() + $item->getRowTotalInclTax());
        }

        return $result;
    }

    /**
     * @param Mage_Sales_Model_Order_Item|Varien_Object $_item
     * @param int $i
     * @param int $item_count
     * @return array
     */
    protected function getBasketItemData($_item, $i, $item_count)
    {
        $_helper = Mage::helper('VerifonePayment');

        if ($_item->getProductType() == 'bundle') {
            $_children = $_item->getChildrenItems();
            $_taxes = array();
            foreach ($_children as $_child) {
                $_taxes[] = $_child->getTaxPercent();
            }

            $average = array_sum($_taxes) / count($_taxes);
            $_item->setTaxPercent($average);
        }

        return array(
            's-t-1-30_bi-name-' . $i => $_helper->_sanitise($_item->getName(), 30),
            'i-t-1-11_bi-unit-count-' . $i => $item_count,
            'l-t-1-20_bi-unit-cost-' . $i => round($_item->getPrice(), 2) * 100,
            'l-t-1-20_bi-net-amount-' . $i => round($_item->getPrice(), 2) * 100 * $item_count,
            'l-t-1-20_bi-gross-amount-' . $i => round($_item->getPriceInclTax(), 2) * 100 * $item_count,
            'i-t-1-4_bi-vat-percentage-' . $i => round($_item->getTaxPercent(), 2) * 100,
            'i-t-1-4_bi-discount-percentage-' . $i => 0,
        );
    }

    protected function getBasketShippingAmountData($i)
    {
        $_order = $this->getOrder();
        $description = preg_replace('/[^A-Za-z0-9\-]/', '', $_order->getShippingDescription());

        return array(
            's-t-1-30_bi-name-' . $i => substr($description, 0, 30),
            'i-t-1-11_bi-unit-count-' . $i => 1,
            'l-t-1-20_bi-unit-cost-' . $i => round($_order->getShippingAmount(), 2) * 100,
            'l-t-1-20_bi-net-amount-' . $i => round($_order->getShippingAmount(), 2) * 100,
            'l-t-1-20_bi-gross-amount-' . $i => $_order->getShippingInclTax() ? round($_order->getShippingInclTax(), 2) * 100 : round($_order->getShippingAmount(), 2) * 100,
            'i-t-1-4_bi-vat-percentage-' . $i =>
                ($_order->getShippingInclTax() - $_order->getShippingAmount()) > 0 ? round(($_order->getShippingInclTax() - $_order->getShippingAmount()) / $_order->getShippingAmount(), 2) * 100 * 100 : 0,
            'i-t-1-4_bi-discount-percentage-' . $i => 0,
        );
    }

    protected function getBasketDiscountData($i, $order_gross_amount, $items_gross_price, $order_net_amount, $items_net_price)
    {
        $_helper = Mage::helper('VerifonePayment');
        $discountDescription = $this->getOrder()->getDiscountDescription();
        $name = strlen($discountDescription) > 0 ? $discountDescription : $_helper->__('Discount');

        return array(
            's-t-1-30_bi-name-' . $i => $_helper->_sanitise($name, 30),
            'i-t-1-11_bi-unit-count-' . $i => 1,
            'l-t-1-20_bi-unit-cost-' . $i => round($order_net_amount - $items_net_price, 0),
            'l-t-1-20_bi-net-amount-' . $i => round($order_net_amount - $items_net_price, 0),
            'l-t-1-20_bi-gross-amount-' . $i => round($order_gross_amount - $items_gross_price, 0),
            'i-t-1-4_bi-vat-percentage-' . $i =>
                round($order_net_amount - $items_net_price, 0) ? round((round($order_gross_amount - $items_gross_price, 0) - round($order_net_amount - $items_net_price, 0)) / round($order_net_amount - $items_net_price, 0), 2) * 100 * 100 : 0,
            'i-t-1-4_bi-discount-percentage-' . $i => 0,
        );
    }

    /**
     * @param $i
     * @param $order_gross_amount
     * @param $order_net_amount
     * @return array
     */
    protected function getCombinedBasketItemsData($i, $order_gross_amount, $order_net_amount)
    {
        $_helper = Mage::helper('VerifonePayment');
        /** @var $_order Mage_Sales_Model_Order */
        $_order = $this->getOrder();
        $incrementId = $_order->getIncrementId();
        // Must be "Tilaus %ORDERNUMBER%" according to docs
        $itemName = "Tilaus $incrementId";

        return array(
            's-t-1-30_bi-name-' . $i => $_helper->_sanitise($itemName, 30),
            'i-t-1-11_bi-unit-count-' . $i => $_order->getTotalItemCount(),
            'l-t-1-20_bi-unit-cost-' . $i => $order_net_amount,
            'l-t-1-20_bi-net-amount-' . $i => $order_net_amount,
            'l-t-1-20_bi-gross-amount-' . $i => $order_gross_amount,
            'i-t-1-4_bi-vat-percentage-' . $i => round(($order_gross_amount - $order_net_amount) / $order_net_amount, 2) * 100 * 100,
            'i-t-1-4_bi-discount-percentage-' . $i => 0,
        );
    }

    protected function getPhoneNumber()
    {
        $session = Mage::getSingleton('customer/session');
        $_helper = Mage::helper('VerifonePayment');

        if (!$session->isLoggedIn() || !$this->getSavedPaymentMethodId()) {

            $_order = $this->getOrder();

            return $_helper->_sanitise($_order->getBillingAddress()->getTelephone(), 30);
        }

        $_paySaved = Mage::getModel('VerifonePayment/db_pay_saved');

        $_card = $_paySaved->getCardByGateMethodId($this->getSavedPaymentMethodId());

        $_additionalData = unserialize($_card->getSerializedData());

        if (isset($_additionalData['s-t-1-30_buyer-phone-number'])) {
            return $_additionalData['s-t-1-30_buyer-phone-number'];
        }

        Mage::throwException($_helper->__('Phone number is necessary to save Credit Card. Please update your default billing address.'));

        return;
    }

    protected function isSendBasketItems()
    {
        return $this->isSendBasketItemsForAll()
            || ($this->isSendBasketItemsForInvoice() && $this->isMethodTypeInvoice());
    }

    /**
     * Verifone has an option to group basket items into 1 combined item.
     * Currently only possible to enable this for invoice payment methods.
     *
     * @return bool
     */
    protected function isCombineBasketItems()
    {
        return ($this->isCombineInvoiceBasketItems() && $this->isMethodTypeInvoice());
    }

    /**
     * If true, basket items are sent for all orders and all payment methods.
     *
     * @return bool
     */
    protected function isSendBasketItemsForAll()
    {
        $basketItemSending = Mage::getStoreConfig('payment/VerifonePayment/basket_item_sending');

        return $basketItemSending == Verifone_VerifonePayment_Model_Adminhtml_System_Config_Source_BasketItemSending::SEND_FOR_ALL;
    }

    /**
     * If true, basket items are sent for orders made with invoice payment methods.
     *
     * @return bool
     */
    protected function isSendBasketItemsForInvoice()
    {
        $basketItemSending = Mage::getStoreConfig('payment/VerifonePayment/basket_item_sending');

        return $basketItemSending == Verifone_VerifonePayment_Model_Adminhtml_System_Config_Source_BasketItemSending::SEND_FOR_INVOICE;
    }

    protected function isCombineInvoiceBasketItems()
    {
        return Mage::getStoreConfig('payment/VerifonePayment/combine_invoice_basket_items');
    }

    protected function isMethodTypeInvoice()
    {
        /** @var $order Mage_Sales_Model_Order */
        $order = $this->getOrder();
        $methodCode = substr($order->getPayment()->getMethod(), 3); // ignore vp_ prefix

        $methodType = Mage::getModel('VerifonePayment/payment')->getMethodType($methodCode);

        return $methodType == "invoice";
    }
}
