<?php

class Verifone_VerifonePayment_Model_Api_S2s_Abstract extends Mage_Payment_Model_Method_Abstract
{
    protected $_url;
    protected $_fields;
    protected $_requestId;
    protected $_response;
    protected $_storeId;

    public function __construct()
    {
        parent::__construct();
        $this->_construct();
    }

    public function _construct()
    {
        parent::_construct();
    }

    protected function getUrls()
    {

        if (!(ini_get('allow_url_fopen'))) {
            Mage::throwException("Configuration error: allow_url_fopen must be enabled");
        }

        return array(
            Mage::getStoreConfig('payment/VerifonePayment/server_url_one', $this->getStoreId()),
            Mage::getStoreConfig('payment/VerifonePayment/server_url_two', $this->getStoreId()),
            Mage::getStoreConfig('payment/VerifonePayment/server_url_three', $this->getStoreId())
        );
    }

    protected function getUrl($_urlIndex = 0)
    {
        if (!Mage::getStoreConfig('payment/VerifonePayment/is_live_mode', $this->getStoreId())) {
            return $this->_url = Mage::getStoreConfig('payment/VerifonePayment/server_url_demo', $this->getStoreId());
        }

        $_urls = $this->getUrls();

        if (isset($_urls[$_urlIndex])) {
            return $this->_url = $_urls[$_urlIndex];
        }

        return false;
    }

    protected function setParametersPost($client, $fields)
    {
        foreach ($fields as $key => $value) {
            $client->setParameterPost($key, $value);
        }

        return $client;
    }

    protected function formatResponse($responseBody)
    {
        return Mage::helper('VerifonePayment')->formatResponse($responseBody);

    }

    protected function getSignatureOne($fields)
    {

        $cryptutils = Mage::helper('VerifonePayment/cryptutils');
        $fields["s-t-256-256_signature-one"] = $cryptutils->generatesignature($cryptutils->formatparameters($fields), $this->getStoreId());

        return $fields;
    }

    public function setRequestId($requestId)
    {
        $this->_requestId = $requestId;
        return $this;
    }

    protected function getRequestId()
    {
        if (is_null($this->_requestId)) {
            $this->setRequestId(time());
        }
        return $this->_requestId;
    }

    public function prepareData()
    {
        if (is_null($this->_fields)) {
            $this->_fields = array();
        }

        $this->_fields['l-f-1-20_request-id'] = $this->getRequestId();

        $this->_fields = array_filter($this->_fields, 'strlen');

        if (!Mage::getStoreConfig('is_local')) {
            $this->_fields = $this->getSignatureOne($this->_fields);
        }

        return $this->_fields;
    }

    public function getPreparedFields()
    {
        if (is_null($this->_fields)) {
            return $this->prepareData();
        }
        return $this->_fields;
    }

    public function clearPreparedFields()
    {
        $this->_fields = null;
        return $this;
    }

    public function sendRequest($_urlIndex = 0)
    {
        try {
            if (Mage::getStoreConfig('is_local')) {
                $_varien = new Varien_Object();
                return $this->setResponse($_varien->setBody(''));
            }

            $_fields = $this->getPreparedFields();

            Mage::dispatchEvent('verifone_s2sgate_send_request_before', array(
                '_class' => get_class($this),
                '_fields' => $_fields
            ));

            $client = new Varien_Http_Client();

            $_url = $this->getUrl($_urlIndex);

            if (!$_url) {
                return $this->setResponse(new Zend_Http_Response(500, array()));
            }

            $client->setUri($_url);
            $client->setMethod(Zend_Http_Client::POST);
            $client->setConfig(array(
                //'maxredirects' => 0,
                'timeout' => 30
            ));
            $client = $this->setParametersPost($client, $_fields);

            $client->setHeaders(array(
                'Content-type' => 'application/x-www-form-urlencoded',
                //'Content-length' => 100,
                'Connection' => 'close'
            ));

            /** @var $_response Zend_Http_Response */
            $_response = $client->request();

            Mage::dispatchEvent('verifone_s2sgate_send_request_after', array(
                '_class' => get_class($this),
                '_response' => $_response,
                '_url' => $_url
            ));

            if ($_response->getStatus() != 200) {

                return $this->sendRequest(++$_urlIndex);
            }

            return $this->setResponse($_response);
        } catch (Exception $e) {
            Mage::getSingleton('core/session')->addError(Mage::helper('VerifonePayment')->__('Something went wrong with payment gateway. Please try again or contact our customer service.'));
            Mage::app()->getFrontController()->getResponse()->setRedirect(Mage::getBaseUrl())->sendResponse();
            exit;
        }
    }

    public function setResponse($_response)
    {
        $this->_response = $_response;
        return $this;
    }

    public function getResponse()
    {
        if (is_null($this->_response)) {
            $this->sendRequest();
        }

        return $this->_response;
    }

    public function getFormatedResponse()
    {
        if (is_null($this->_response)) {
            $this->getResponse();
        }

        return $this->formatResponse($this->_response->getBody());
    }

    public function setStoreId($storeId)
    {
        $this->_storeId = $storeId;
        return $this;
    }

    public function getStoreId()
    {
        return $this->_storeId;
    }
}
