<?php

class Verifone_VerifonePayment_Model_Api_S2s_ListSavedPaymentMethods extends Verifone_VerifonePayment_Model_Api_S2s_Abstract
{
    protected $_order;
    protected $_customer;
    protected $_additionalData;

    public function prepareData()
    {
        $_customer = $this->getCustomer();
        $_helper = Mage::helper('VerifonePayment');
        $_helperConfiguration = Mage::helper('VerifonePayment/configuration');

        $_fields = array(
            's-f-1-30_operation' => 'list-saved-payment-methods', //this value is obligatory but it would be filled in every method
            't-f-14-19_request-timestamp' => gmdate('Y-m-d H:i:s'),
            's-f-1-36_merchant-agreement-code' => $_helperConfiguration->getMerchantAgreement(),
            's-f-1-30_software' => 'Magento',
            's-f-1-10_software-version' => $_helper->_sanitise(Mage::getVersion(), 10, true),

            //important difference
            'i-f-1-11_interface-version' => '4',

            's-f-1-30_buyer-first-name' => substr($_customer->getFirstname(), 0, 30),
            's-f-1-30_buyer-last-name' => substr($_customer->getLastname(), 0, 30),
            's-f-1-100_buyer-email-address' => $_helper->_sanitise($_customer->getEmail(), 100),

            's-t-1-30_recurring-payment-subscription-code' => $this->getProfileReferenceId()

        );

        $this->_fields = array_merge($_fields, $this->getBuyerDataFromOrder());

        $this->mergeAdditionalData();

        return parent::prepareData();
    }

    public function getBuyerDataFromOrder()
    {
        if (!$this->_order) {
            return array();
        }
        $_helper = Mage::helper('VerifonePayment');

        $billingAddress = $this->_order->getBillingAddress();
        $telephone = $billingAddress->getTelephone();
        $_fields = array();

        $_fields['s-f-1-30_buyer-first-name'] = substr($billingAddress->getFirstname(), 0, 30);
        $_fields['s-f-1-30_buyer-last-name'] = substr($billingAddress->getLastname(), 0, 30);
        $_fields['s-t-1-30_buyer-phone-number'] = $_helper->_sanitise($telephone, 30);
        $_fields['s-f-1-100_buyer-email-address'] = $_helper->_sanitise($billingAddress->getData('email'), 100);

        $shippingAddress = $this->_order->getShippingAddress();

        if ($shippingAddress) {
            $_fields['s-t-1-30_delivery-address-line-one'] = $_helper->_sanitise($shippingAddress->getStreet1(), 30);
            $_fields['s-t-1-30_delivery-address-line-two'] = $_helper->_sanitise($shippingAddress->getStreet2(), 30) ? $_helper->_sanitise($shippingAddress->getStreet2(), 30) : $_helper->_sanitise($shippingAddress->getRegion(), 30);
            $_fields['s-t-1-30_delivery-address-line-three'] = $_helper->_sanitise($shippingAddress->getStreet2(), 30) ? $_helper->_sanitise($shippingAddress->getRegion(), 30) : '';
            $_fields['s-t-1-30_delivery-address-city'] = $_helper->_sanitise($shippingAddress->getCity(), 30);
            $_fields['s-t-1-30_delivery-address-postal-code'] = $_helper->_sanitise($shippingAddress->getPostcode(), 30);
            $_fields['i-t-1-3_delivery-address-country-code'] = $_helper->convertCountryCode2Numeric(
                $shippingAddress->getCountry()
            );
        }

        return $_fields;
    }

    public function setOrder($_order)
    {
        if ($_order) {
            $this->_order = $_order;
        }

        return $this;
    }

    protected function getOrder()
    {
        return $this->_order;
    }

    public function setAdditionalData($_data)
    {
        $this->_additionalData = $_data;
        return $this;
    }

    protected function getAdditionalData()
    {
        return !is_null($this->_additionalData) && is_array($this->_additionalData) ? $this->_additionalData : array();
    }

    protected function getCustomer()
    {
        if (is_null($this->_customer)) {
            $session = Mage::getSingleton('customer/session');
            $_customer = Mage::getModel('customer/customer')->load($session->getId());

            $this->_customer = $_customer ? $_customer : null;
        }

        return $this->_customer;
    }

    protected function mergeAdditionalData()
    {
        $_data = $this->getAdditionalData();

        foreach ($_data as $_directive => $_value) {
            $this->_fields[$_directive] = $_value;
        }

        return $this;
    }

    protected function getProfileReferenceId()
    {
        $_order = $this->getOrder();
        if (!is_null($_order)) {
            return $_order->getProfileReferenceId();
        }

        return '';
    }

}
