<?php

class Verifone_VerifonePayment_Model_Observer_Cron extends Verifone_VerifonePayment_Model_Observer
{
    public function chargeRecurringProfiles(Mage_Cron_Model_Schedule $schedule)
    {
        $db = Mage::getModel('VerifonePayment/db_pay_options');
        $profiles = $db->getRecurringProfilesForPaymentProcess();

        foreach ($profiles as $profile) {
            $recurring = $this->getMethodNamespace($profile->getMethodCode());
            $orders = $profile->getResource()->getChildOrderIds($profile);
            $countBillingCycling = count($orders);

            $profile = $this->formatProfile($profile);

            if ($recurring->chargeRecurringProfile($profile, true)) {
                $countBillingCycling++;
            } else {
                $storeId = $profile->getStoreId();
                $templateId = Mage::getStoreConfig('payment/VerifonePayment/payment_failed_email_template', $storeId);
                $email_to = $profile->getOrderInfo()['customer_email'];
                $customer_name = $profile->getOrderInfo()['customer_firstname'] . ' ' . $profile->getOrderInfo()['customer_lastname'];
                $email_template = Mage::getModel('core/email_template')->loadDefault($templateId);

                $email_template_variables = array(
                    'customer_name' => $customer_name,
                    'order_no' => Mage::getModel('sales/order')->load(end($orders))->getIncrementId(),
                    'amount' => $profile->getBillingAmount() + $profile->getShippingAmount() + $profile->getTaxAmount(),
                    'date' => Mage::getModel('core/date')->date('Y.m.d'),
                );

                $sender_name = Mage::getStoreConfig(Mage_Core_Model_Store::XML_PATH_STORE_STORE_NAME);
                $sender_email = Mage::getStoreConfig('trans_email/ident_general/email');
                $email_template->setSenderName($sender_name);
                $email_template->setSenderEmail($sender_email);

                $email_template->send($email_to, $customer_name, $email_template_variables);
            }

            if ($profile->getPeriodMaxCycles() && $countBillingCycling >= $profile->getPeriodMaxCycles()) {
                $profile->setState(Mage_Sales_Model_Recurring_Profile::STATE_SUSPENDED);
                $profile->save();
            }
        }
    }

    public function checkIfCardIsExpired(Mage_Cron_Model_Schedule $schedule)
    {
        $db = Mage::getModel('VerifonePayment/db_pay_options');
        $profiles = $db->getRecurringProfilesForExpireCheck();

        foreach ($profiles as $profile) {
            $profile = $this->formatProfile($profile);
            $orders = $profile->getResource()->getChildOrderIds($profile);

            if (!empty($orders)) {
                $_order = Mage::getModel('sales/order')->load($orders[0]);
                $_order->setProfileReferenceId($profile->getReferenceId());

                $_apiListSavedPaymentMethods = Mage::getModel('VerifonePayment/api_s2s_listSavedPaymentMethods');
                $_savedPaymentMethodsResponse = $_apiListSavedPaymentMethods->setOrder($_order)
                    ->sendRequest()
                    ->getFormatedResponse();

                /* @var $date Mage_Core_Model_Date */
                $date = Mage::getModel('core/date');
                $expireDate = Mage::helper('VerifonePayment')->getPaymentMethodExpireDate($_savedPaymentMethodsResponse, $_order->getPayment()->getMethod());
                if (is_null($expireDate)) {
                    return false;
                }
                $expireDate->add(new DateInterval('P1M'));
                $now = date_create($date->date('Y-m-d'));
                $interval = date_diff($now, $expireDate);

                /**
                 * interwal is 7 days before card would be expired
                 * invert is 1 when expire_date < now
                 * */
                if ($interval->d == 7 && !$interval->invert) {
                    $storeId = $profile->getStoreId();
                    $templateId = Mage::getStoreConfig('payment/VerifonePayment/expire_card_email_template', $storeId);
                    $email_to = $profile->getOrderInfo()['customer_email'];
                    $customer_name = $profile->getOrderInfo()['customer_firstname'] . ' ' . $profile->getOrderInfo()['customer_lastname'];
                    $email_template = Mage::getModel('core/email_template')->loadDefault($templateId);

                    $email_template_variables = array(
                        'shop_url' => Mage::helper('core/url')->getHomeUrl(),
                        'store_name' => Mage::app()->getStore()->getName()
                    );

                    $sender_name = Mage::getStoreConfig(Mage_Core_Model_Store::XML_PATH_STORE_STORE_NAME);
                    $sender_email = Mage::getStoreConfig('trans_email/ident_general/email');
                    $email_template->setSenderName($sender_name);
                    $email_template->setSenderEmail($sender_email);

                    $email_template->send($email_to, $customer_name, $email_template_variables);

                } else if (($interval->d == 0 && !$interval->invert) || $interval->invert) {
                    $profile->setState(Mage_Sales_Model_Recurring_Profile::STATE_EXPIRED);
                    $profile->save();
                }
            }
        }

        return $this;
    }

    public function checkPaymentStatusForNewOrders(Mage_Cron_Model_Schedule $schedule)
    {
        $ordersCollection = $this->getOrdersToCheck();

        $ordersCollection->getSelect()
            ->where('created_at > ?', new Zend_Db_Expr('NOW() - INTERVAL 3 HOUR'));

        $this->checkPaymentStatus($ordersCollection);
    }

    public function checkPaymentStatusForOldOrders(Mage_Cron_Model_Schedule $schedule)
    {
        $ordersCollection = $this->getOrdersToCheck();
        $ordersCollection->getSelect()
            ->where('created_at > ?', new Zend_Db_Expr('NOW() - INTERVAL 3 DAY'))
            ->where('created_at <= ?', new Zend_Db_Expr('NOW() - INTERVAL 3 HOUR'));

        $this->checkPaymentStatus($ordersCollection);

    }

    private function checkPaymentStatus(Mage_Sales_Model_Resource_Order_Collection $ordersCollection)
    {
        /** @var Mage_Sales_Model_Order $_order */
        foreach ($ordersCollection as $_order) {
            try {
                Mage::getSingleton('VerifonePayment/payment')->getTransactions(
                    $_order->getIncrementId(),
                    $update = true,
                    $_cron = true
                );
            } catch (Exception $e) {
                Mage::logException($e);
            }
        }
        return;
    }
}