<?php

class Verifone_VerifonePayment_Model_Observer_Events extends Verifone_VerifonePayment_Model_Observer
{
    public function saveCreditCard(Varien_Event_Observer $_observer)
    {
        $_fields = $_observer->getEvent()->getData('_fields');

        if (!Mage::getStoreConfig('payment/VerifonePayment/allow_to_save_cc')
            || !isset($_fields['i-t-1-1_save-payment-method'])
            || !in_array($_fields['i-t-1-1_save-payment-method'], array('1', '2'))
        ) {
            return;
        }

        Mage::getModel('VerifonePayment/db_pay_saved')->savePaymentMethod($_observer->getEvent()->getData('_fields'));
    }

    public function updateCreditCard(Varien_Event_Observer $_observer)
    {
        Mage::getModel('VerifonePayment/db_pay_saved')
            ->updatePaymentMethods();

    }

    public function logRequest(Varien_Event_Observer $_observer)
    {

        return Mage::log($_observer->getEvent()->getData(), null, 'verifone.log', true);
    }

    public function logResponse(Varien_Event_Observer $_observer)
    {
        $_response = $_observer->getEvent()->getData('_response');
        $responseBody = $_response && !is_array($_response) ? $_response->getBody() : array();
        $_formatedResponse = Mage::helper('VerifonePayment')->formatResponse($responseBody);

        $result = $_observer->getEvent()->getData();

        if (!empty($_formatedResponse)) {
            $result['_formatedResponse'] = $_formatedResponse;
        }

        $_url = $_observer->getEvent()->getData('_url');
        if (!empty($_url)) {
            $result['_url'] = $_url;
        }

        return Mage::log($result, null, 'verifone.log', true);
    }

    public function sales_model_service_quote_submit_after(Varien_Event_Observer $observer)
    {

        $payment = $observer->getQuote()->getPayment();

        if ($payment['method'] == 'VerifonePayment' || substr($payment['method'], 0, 3) == 'vp_') {
            // Activate the quote
            $observer->getQuote()->setIsActive(true);
        }
    }

    public function sales_quote_payment_import_data_before(Varien_Event_Observer $observer)
    {
        $_additional = array();
        $_method = $observer->getInput()['method'];
        if (isset($observer->getInput()['i-t-1-1_save-payment-method'])) {
            $_additional = array_merge($_additional, array(
                'i-t-1-1_save-payment-method' => '1'
            ));
        }

        if (isset($observer->getInput()['s-t-1-30_payment-method-code'])) {
            $_additional = array_merge($_additional, array(
                's-t-1-30_payment-method-code' => $observer->getInput()['s-t-1-30_payment-method-code']
            ));
        }

        if (isset($observer->getInput()['l-t-1-20_saved-payment-method-id'])) {
            $_additional = array_merge($_additional, array(
                'l-t-1-20_saved-payment-method-id' => $observer->getInput()['l-t-1-20_saved-payment-method-id']
            ));
        }

        if (empty($_additional)) {
            return;
        }

        $observer
            ->getPayment()
            ->setAdditionalInformation($_additional);


        if (substr($_method, 0, 13) == 'vp-recurring_') {
            $observer
                ->getPayment()
                ->setMethod($_method)
                ->save();

        }
    }

    public function checkout_submit_all_after(Varien_Event_Observer $observer)
    {
        $_quote = $observer->getQuote();
        $payment = $_quote->getPayment();

        if (substr($payment['method'], 0, 13) == 'vp-recurring_') {
            Mage::getSingleton('checkout/type_onepage')
                ->getCheckout()
                ->setRedirectUrl(Mage::getUrl('VerifonePayment/redirect'));
        }
    }

    public function checkoutTypeOnepageSaveOrderAfter(Varien_Event_Observer $_observer)
    {
        // Due to PSD/2 regulation payment must be redirect to the payment provider page.
        // Is not possible to make payment request by S2S integration
        return;
    }

    public function addCheckingStatusButton(Varien_Event_Observer $_observer)
    {
        $block = Mage::app()->getLayout()->getBlock('sales_order_edit');
        if (!$block) {
            return $this;
        }
        $_order = Mage::registry('current_order');

        $_paymentMethod = $_order->getPayment()->getMethodInstance();

        if (!$_paymentMethod instanceof Verifone_VerifonePayment_Model_Payment) {
            return $this;
        }

        $url = Mage::getSingleton('adminhtml/url')->getUrl(
            '*/verifone_sales_order/checkStatus',
            array(
                'order_increment_id' => $_order->getIncrementId(),
                'order_id' => $_order->getId()
            )
        );

        $block->addButton('verifone_checkStatus', array(
            'label' => Mage::helper('VerifonePayment')->__('Check payment status'),
            'onclick' => 'setLocation(\'' . $url . '\')',
            'class' => 'go'
        ));
        return $this;
    }

    public function saveMaskedPanNumber(Varien_Event_Observer $_observer)
    {
        if(Mage::getStoreConfig('payment/VerifonePayment/save_masked_pan_number')) {
            $_response = $_observer->getEvent()->getData('_response');
            if(!is_array($_response)) {
                $_response = Mage::helper('VerifonePayment')->formatResponse($_response->getBody());
            }
            if(isset($_response['s-f-1-36_order-number'])
                && isset($_response['i-t-2-2_card-pan-last2'])
                && !empty($_response['i-t-2-2_card-pan-last2'])
                && isset($_response['i-t-6-6_card-pan-first6'])
                && !empty($_response['i-t-6-6_card-pan-first6'])
            ) {
                $maskedPan = $_response['i-t-6-6_card-pan-first6'] . '********' . $_response['i-t-2-2_card-pan-last2'];

                $coreResource = \Mage::getModel('core/resource');
                $connection = $coreResource->getConnection('core_write');
                $connection->update('sales_flat_order', ['masked_pan_number' => $maskedPan], 'increment_id = '. $connection->quote($_response['s-f-1-36_order-number']));
            }
        }
    }
}
