<?php

class Verifone_VerifonePayment_Model_Payment extends Mage_Payment_Model_Method_Abstract
{
    protected $_code = 'VerifonePayment';
    protected $_formBlockType = 'VerifonePayment/payment_methods_form';
    protected $_infoBlockType = 'VerifonePayment/payment_methods_info';
    protected $_paymentMethod = 'shared';
    protected $_canCapture = true;
    protected $_isCard = false;
    protected $_orderPlaceRedirectUrl;

    public function __construct()
    {
        parent::__construct();

        if (!Mage::helper('core')->isModuleEnabled('Verifone_Refund') && !class_exists('Verifone_Refund_Model_Observer')) {
            $this->_canRefund = true;
            $this->_canRefundInvoicePartial = true;
        }
    }

    public function getCheckout()
    {
        return Mage::getSingleton('checkout/session');
    }

    public function cartHasItems()
    {
        if ($this->getCheckout()) {
            if ($this->getCheckout()->getQuote()->hasItems()) {
                return true;
            }
        }
        return false;
    }

    public function resetOrderPlaceRedirectUrl()
    {
        $this->setOrderPlaceRedirectUrl('');

        return $this;
    }

    public function setOrderPlaceRedirectUrl($_url)
    {
        $this->_orderPlaceRedirectUrl = $_url;

        return $this;
    }

    public function getOrderPlaceRedirectUrl()
    {
        if (!is_null($this->_orderPlaceRedirectUrl)) {
            return $this->_orderPlaceRedirectUrl;
        }

        return Mage::getUrl('VerifonePayment/redirect');
    }

    public function isAvailable($quote = null)
    {
        if (!Mage::getStoreConfig('payment/VerifonePayment/active')) {
            return false;
        }

        $selectedPayments = $this->getAllSelectedPayments();

        $collection = Mage::getModel('VerifonePayment/db_pay_options')->getCollection()
            ->addFieldToFilter('active', array('eq' => '1'));
        $gateCode = $this->getGateCode(); //remove "vp_" prefix from the code

        foreach ($collection as $_item) {

            if (!in_array($gateCode, $selectedPayments)) {
                continue;
            }

            if ($gateCode == $_item->getCode()) {
                return true;
            }
        }
        return false;
    }

    public function getAllSelectedPayments()
    {
        $_cardsPayments = unserialize(Mage::getStoreConfig('payment/VerifonePayment/cardpaymentsgroup_array'));
        $_customGroupsPayments = unserialize(Mage::getStoreConfig('payment/VerifonePayment/paymentsgroups_array'));

        $groupedPayments = array_merge($_customGroupsPayments, $_cardsPayments);

        $_result = array();
        foreach ($groupedPayments as $_group) {
            if (isset($_group['payments'])) {
                $_result = array_merge($_result, $_group['payments']);
            }
        }

        return $_result;
    }

    public function getTitle()
    {
        $gateCode = $this->getGateCode(); //remove "vp_" prefix from the code

        if ($gateCode) {
            $dbCollection = Mage::getModel('VerifonePayment/db_pay_options')->getCollection()
                ->addFieldToFilter('code', array('eq' => $gateCode));

            $_item = $dbCollection->getFirstItem();
            return Mage::helper('VerifonePayment')->__($_item->getName());
        }

        return parent::getTitle();
    }

    public function capture(Varien_Object $payment, $amount)
    {
        if ($payment->getStatus() != self::STATUS_APPROVED) {
            $payment->setStatus(self::STATUS_APPROVED)
                ->setTransactionId($this->getTransactionId())
                ->setIsTransactionClosed(0);
        }
        return $this;
    }

    public function getGateCode($code = null)
    {
        if ($code) {
            return substr($code, 3);
        }

        if ($this->getCode() == 'VerifonePayment') {
            return $this->getCode();
        }

        return substr($this->getCode(), 3);
    }

    public function getMethodType($_gateCode = null)
    {
        if (is_null($_gateCode)) {
            $_gateCode = $this->getGateCode();
        }

        $collection = Mage::getModel('VerifonePayment/db_pay_options')->getCollection()
            ->addFieldToFilter('code', array('eq' => $_gateCode));

        return strtolower($collection->getFirstItem()->getType());
    }

    public function isCard()
    {
        return $this->_isCard;
    }

    protected function getTransactionsFromGate($_orderIncrementId, $storeId = null)
    {
        $_apiListTransactionNumbers = Mage::getModel('VerifonePayment/api_s2s_listTransactionNumbers');
        $_listTransactionNumbersResponse = $_apiListTransactionNumbers
            ->setOrderNumber($_orderIncrementId)
            ->setStoreId($storeId)
            ->sendRequest()
            ->getFormatedResponse();

        if (isset($_listTransactionNumbersResponse['s-f-1-30_error-message'])) {
            return Mage::throwException($_listTransactionNumbersResponse['s-f-1-30_error-message']);
        }

        return $_listTransactionNumbersResponse;
    }

    protected function getPaymentStatus($_transactionData = array(), $storeId)
    {
        $_apiGetPaymentStatus = Mage::getModel('VerifonePayment/api_s2s_getPaymentStatus');

        $_getPaymentStatusResponse = $_apiGetPaymentStatus
            ->setPaymentMethodCode($_transactionData['s-f-1-30_payment-method-code'])
            ->setTransactionNumber($_transactionData['l-f-1-20_transaction-number'])
            ->setStoreId($storeId)
            ->sendRequest()
            ->getFormatedResponse();

        if (isset($_getPaymentStatusResponse['s-f-1-30_error-message'])) {
            return Mage::throwException($_getPaymentStatusResponse['s-f-1-30_error-message']);
        }

        $_transaction = array();

        if (isset($_getPaymentStatusResponse['l-f-1-20_transaction-number']) && $_getPaymentStatusResponse['l-f-1-20_transaction-number']) {

            $_helper = Mage::helper('VerifonePayment');

            $_transaction['transaction_id'] = $_getPaymentStatusResponse['l-f-1-20_transaction-number'];
            $_transaction['transaction_type'] = $_helper->getTransactionTypeFromMap($_getPaymentStatusResponse['s-f-1-30_payment-status-code']);
            $_transaction['s-f-1-30_payment-status-code'] = $_getPaymentStatusResponse['s-f-1-30_payment-status-code'];
            $_transaction['transaction_message'] = $_helper->getVerifoneStatusDescription(
                $_getPaymentStatusResponse['s-f-1-30_payment-status-code'],
                $this->getMethodType($_getPaymentStatusResponse['s-f-1-30_payment-method-code'])
            );
            $_transaction['order_increment_id'] = $_getPaymentStatusResponse['s-f-1-36_order-number'];
            $_transaction['l-f-1-20_order-gross-amount'] = $_getPaymentStatusResponse['l-f-1-20_order-gross-amount'];


        }

        return $_transaction;
    }

    public function getTransactions($_orderIncrementId, $_update = false, $_cron = false)
    {
        $_helper = Mage::helper('VerifonePayment');
        $_transactions = $_transaction = array();

        /** @var Mage_Sales_Model_Order $_order */
        $_order = Mage::getModel('sales/order')->loadByIncrementId($_orderIncrementId);

        if (! in_array($_order->getStatus(), array($_order::STATE_NEW, $_order::STATE_PENDING_PAYMENT, 'pending', $_order::STATE_CANCELED))) {
            return;
        }

        if ($this->transactionsWasCheckedToday($_order)) {
            return;
        }

        $response = $this->getTransactionsFromGate($_orderIncrementId, $_order->getStoreId());

        $_totalPaid = 0;
        foreach ($response as $_responseKey => $_responseValue) {
            if (strpos($_responseKey, 'l-f-1-20_transaction-number') !== false) {
                $_transactionCode = str_replace('l-f-1-20_transaction-number', 's-f-1-30_payment-method-code', $_responseKey);

                $_transactionData = array(
                    'l-f-1-20_transaction-number' => $_responseValue,
                    's-f-1-30_payment-method-code' => $response[$_transactionCode]
                );

                $_transaction = array_merge($this->getPaymentStatus($_transactionData, $_order->getStoreId()), $_transactionData);

                if (!empty($_transaction)) {
                    $_transactions[] = $_transaction;

                    $_totalPaid += $_transaction['l-f-1-20_order-gross-amount'];

                    if ($_update) {
                        $this->updateTransaction($_order, $_transaction);
                    }
                }
            }
        }

        if (!empty($_transaction) && $_update) {
            $_state = $_helper->getOrderStatusFromMap($_transaction['s-f-1-30_payment-status-code']);

            if ($_state == $_order->getState()) {
                return;
            }

            if ($_status = $_state) {
                $_comment = $_helper->__('Order status is now [%s] because payment for this order was found.', $_state);

                if (in_array($_state, array(Mage_Sales_Model_Order::STATE_CANCELED, Mage_Sales_Model_Order::STATE_PENDING_PAYMENT))) {
                    $_comment = $_helper->__('Order status is now [%s] because payment for this order was NOT found.', $_state);
                } else {
                    // we have payment, but for some reason(s) order could be canceled
                    if ($_order->getStatus() == Mage_Sales_Model_Order::STATE_CANCELED) {
                        // Have to uncancel all order items so invoice can be generated.
                        Mage::helper('VerifonePayment/order')->uncancelOrder($_order);
                    }
                }

                if (!$_cron) {
                    Mage::getSingleton('core/session')->addSuccess($_comment);
                }

                $_order->setState($_state, $_status, $_comment);

                $_order->save();

                if ($_order->canInvoice()) {
                    $invoice = $_order->prepareInvoice();
                    $invoice->register()->capture();
                    Mage::getModel('core/resource_transaction')->addObject($invoice)->addObject($invoice->getOrder())->save();
                }
            }
        }

        return $_transactions;
    }

    /**
     * @param Mage_Sales_Model_Order $_order
     * @return bool
     */
    protected function transactionsWasCheckedToday($_order)
    {
        $transactions = Mage::getResourceModel('sales/order_payment_transaction_collection');
        $transactions
            ->addOrderIdFilter($_order->getId())
            ->addOrderInformation(array('increment_id'))
            ->addPaymentInformation(array('method'));

        $date = new DateTime(Mage::getModel('core/date')->date('Y-m-d'));

        /** @var Mage_Sales_Model_Order_Payment_Transaction $transaction */
        foreach ($transactions as $_transaction) {
            $rawData = $_transaction->getAdditionalInformation(Mage_Sales_Model_Order_Payment_Transaction::RAW_DETAILS);

            if ($rawData && !empty($rawData) && isset($rawData['date_of_check'])) {
                $_dateOfCheck = new DateTime($rawData['date_of_check']);

                if ($date->diff($_dateOfCheck)->days == 0) {
                    return true;
                }
            }
        }

        return false;
    }


    /**
     * @param Mage_Sales_Model_Order $_order
     * @param array $_data
     * @throws Exception
     */
    public function updateTransaction($_order, $_data)
    {
        $_helper = Mage::helper('VerifonePayment');

        if (empty($_data)) {
            return;
        }

        /** @var Mage_Sales_Model_Order_Payment $_payment */
        $_payment = $_order->getPayment();

        /** @var Mage_Sales_Model_Order_Payment_Transaction $_transaction */
        $_transaction = $_payment->getTransaction($_data['transaction_id']);

        if (!$_transaction) {
            $_transaction = $_payment->getTransaction('verifone_' . $_data['transaction_id']);
        }

        $date = Mage::getModel('core/date')->date('Y-m-d');

        if ($_transaction) {
            $_transaction
                ->setTxnId($_data['transaction_id'])
                ->setTxnType($_data['transaction_type'])
                ->setAdditionalInformation(
                    Mage_Sales_Model_Order_Payment_Transaction::RAW_DETAILS,
                    array(
                        's-f-1-30_payment-status-code' => $_data['s-f-1-30_payment-status-code'],
                        'message' => $_data['transaction_message'],
                        'date_of_check' => $date
                    ))
                ->save();

            return;
        }

        $_payment->setTransactionAdditionalInfo(
            Mage_Sales_Model_Order_Payment_Transaction::RAW_DETAILS,
            array(
                's-f-1-30_payment-status-code' => $_data['s-f-1-30_payment-status-code'],
                'message' => $_data['transaction_message'],
                'l-f-1-20_order-gross-amount' => $_data['l-f-1-20_order-gross-amount'],
                'date_of_check' => $date
            )
        );

        if (empty($_payment->getAdditionalInformation('s-f-1-30_payment-method-code'))) {
            $_payment->setAdditionalInformation('s-f-1-30_payment-method-code', $_data['s-f-1-30_payment-method-code']);
        }

        if (empty($_order->getExtOrderId())) {
            $_order->setData('ext_order_id', $_data['transaction_id']);
        }

        $_payment
            ->setTransactionId($_data['transaction_id'])
            ->setIsTransactionClosed(1)
            ->addTransaction($_data['transaction_type'])
            ->save();

        if ($_data['transaction_message']) {
            $_order->addStatusHistoryComment($_data['transaction_message']);
        }

        $_order->save();
    }

    /**
     * @param Varien_Object $payment
     * @param float $amount
     * @return $this
     * @throws Exception
     */
    public function refund(Varien_Object $payment, $amount)
    {

        /** @var Mage_Sales_Model_Order $order */
        $order = $payment->getOrder();
        $total = round($amount, 2);

        $helper = Mage::helper('VerifonePayment');

        if ($order->getId() && $total > 0) {
            $payment = $order->getPayment();

            $paymentMethodCode = $payment->getAdditionalInformation('s-f-1-30_payment-method-code');

            if ($paymentMethodCode === null) {
                $paymentMethodCode = $payment->getAdditionalInformation('s-t-1-30_payment-method-code');
            }

            if ($order->getExtOrderId() && $paymentMethodCode) {
                $order->setData('verifone_payment_method', $paymentMethodCode);
                $apiRefundPayment = Mage::getModel('VerifonePayment/api_s2s_refundPayment');

                $response = $apiRefundPayment->setOrder($order)
                    ->setPrice($total * 100)
                    ->sendRequest()
                    ->getFormatedResponse();

                if (!isset($response['s-f-1-30_error-message'])) {

                    $amountTxt = sprintf($order->getOrderCurrency()->getOutputFormat(), $total);
                    $comment = sprintf($helper->__('Refunded amount of %s via Verifone Payment service was done successfully.'), $amountTxt);

                    $order->addStatusHistoryComment($comment);
                    $order->save();
                    return $this;
                } else {
                    throw new Exception(sprintf($helper->__('Is not possible to make refund now. Error message: %s'), $response['s-f-1-30_error-message']));
                }
            } else {
                throw new Exception($helper->__('Is not possible to make online refund'));
            }

        } else {
            throw new Exception($helper->__('Refund has missing data.'));
        }

        return $this;
    }
}
