<?php

class Verifone_VerifonePayment_RedirectController extends Mage_Core_Controller_Front_Action
{
    const RETRY_DELAY_IN_SECONDS = 2;
    const RETRY_MAX_ATTEMPTS = 5;

    protected function _expireAjax()
    {
        $cartHasItems = Mage::getModel('VerifonePayment/payment')->cartHasItems();
        if (!$cartHasItems) {
            $this->getResponse()->setHeader('HTTP/1.1', '403 Session Expired');
            exit;
        }
    }

    protected function _saveInvoice(Mage_Sales_Model_Order $order)
    {
        if ($order->canInvoice()) {
            $invoice = $order->prepareInvoice();
            $invoice->register()->capture();
            Mage::getModel('core/resource_transaction')->addObject($invoice)->addObject($invoice->getOrder())->save();
            return true;
        }
        return false;
    }

    protected function _defaultAction()
    {
        $_request = $this->getRequest();
        $_signedFormData = $_request->getParams();

        Mage::dispatchEvent('verifone_paymentinterface_send_request_after', array(
            '_class' => get_class($this),
            '_response' => $_signedFormData
        ));

        unset($_signedFormData["s-t-1-40_shop-order__phase"]);
        unset($_signedFormData["s-t-256-256_signature-one"]);
        unset($_signedFormData["s-t-256-256_signature-two"]);
        $_cryptUtils = Mage::helper('VerifonePayment/cryptutils');
        $_signatureValid = @$_cryptUtils->verifysignature($_cryptUtils->formatparameters($_signedFormData),
            $_request->getParam('s-t-256-256_signature-one'));
        if (!$_signatureValid) {
            Mage::getSingleton('core/session')->addError(Mage::helper('VerifonePayment')->__('Payment interface error'));
            return $this->_redirect('checkout/onepage');
        }

        $_order = Mage::getModel('sales/order')->load(Mage::getSingleton('checkout/session')->getLastOrderId());

        if ($_transactionId = $_request->getParam('l-f-1-20_transaction-number')) {
            $payment = $_order->getPayment();

            if (!$payment) {
                return $this->_redirect('checkout/onepage');
            }

            $payment
                ->setTransactionId($_transactionId)
                ->setIsTransactionClosed(1)
                ->addTransaction(Mage_Sales_Model_Order_Payment_Transaction::TYPE_PAYMENT);

            $payment->save();
        }

        if (Mage_Sales_Model_Order::STATE_CANCELED !== $_order->getState()) {
            $_order->cancel();

            $history = Mage::helper('VerifonePayment')->__('Payment was canceled. Cancel reason: ') . $_request->getParam('s-t-1-30_cancel-reason');
            $_order->addStatusToHistory($_order->getStatus(), $history);
        }

        $_order->save();

        return $this->_redirect('checkout/onepage');
    }

    protected function _handleSuccess($delayedSuccess = false)
    {
        /* @var Verifone_VerifonePayment_Model_Resource_Order_Process_Status $orderLocker */
        $orderLocker = Mage::getResourceModel('VerifonePayment/order_process_status');
        $_request = $this->getRequest();
        $_signedFormData = $_request->getParams();

        Mage::dispatchEvent('verifone_paymentinterface_send_request_after', array(
            '_class' => get_class($this),
            '_response' => $_signedFormData
        ));

        if(empty($_request->getParam('s-f-1-36_order-number', null))) {
            return $this->_redirect('checkout/onepage');
        }

        $attempts = 0;

        while (!$orderLocker->lockOrder((int)$_request->getParam('s-f-1-36_order-number')) && $attempts < self::RETRY_MAX_ATTEMPTS) {
            sleep(self::RETRY_DELAY_IN_SECONDS);
            ++$attempts;
        }

        try {
            Mage::getSingleton('checkout/session')->getQuote()->setIsActive(false)->save();  // empty cart

            $order = Mage::getModel('sales/order')->loadByIncrementId((int)$_request->getParam('s-f-1-36_order-number'));
            unset($_signedFormData["s-t-1-40_shop-receipt__phase"]);
            unset($_signedFormData["s-t-256-256_signature-one"]);
            unset($_signedFormData["s-t-256-256_signature-two"]);
            $cryptutils = $cryptutils = Mage::helper('VerifonePayment/cryptutils');
            $signature_valid = @$cryptutils->verifysignature($cryptutils->formatparameters($_signedFormData),
                $_request->getParam('s-t-256-256_signature-one'));
            if ($signature_valid
                && $_request->getParam('l-f-1-20_transaction-number')
                && abs($_request->getParam('l-f-1-20_order-gross-amount') - (round($order->getGrandTotal(),
                            2) * 100)) < 1
                && !$_request->getParam('s-t-1-30_cancel-reason')
            ) {
                // the order might be handled already
                if ($order->getBaseTotalDue()) {
                    $trans_id = preg_replace("/[^0-9]+/", "", $_request->getParam('l-f-1-20_transaction-number'));
                    $_transactionId = $_request->getParam('l-f-1-20_transaction-number');

                    // we have payment, but for some reason(s) order could be canceled
                    if ($order->getStatus() == Mage_Sales_Model_Order::STATE_CANCELED) {
                        // Have to uncancel all order items so invoice can be generated.
                        Mage::helper('VerifonePayment/order')->uncancelOrder($order);
                    }

                    $order->setData('ext_order_id', $trans_id);
                    $order->setData('verifone_payment_method',
                        preg_replace("/[^0-9-\w\s]+/", "", $_request->getParam('s-f-1-30_payment-method-code')));
                    $payment = $order->getPayment();
                    $payment
                        ->setTransactionId($_transactionId)
                        ->setIsTransactionClosed(1)
                        ->addTransaction(Mage_Sales_Model_Order_Payment_Transaction::TYPE_CAPTURE);

                    $payment->setAdditionalInformation('s-f-1-30_payment-method-code', preg_replace("/[^0-9-\w\s]+/", "", $_request->getParam('s-f-1-30_payment-method-code')));

                    $order->addStatusHistoryComment($this->__('Payment %s captured', $_transactionId),
                        Mage_Sales_Model_Order::STATE_PROCESSING);
                    $order->setState(Mage_Sales_Model_Order::STATE_PROCESSING);

                    $order->save();
                    $payment->save();

                    $this->_saveInvoice($order);

                    if (!$order->getEmailSent()) {
                        $order->sendNewOrderEmail();  // in case of multiple returns we don't want email each time
                    }
                }
                $orderLocker->unlockOrder((int)$_request->getParam('s-f-1-36_order-number'));
                if ($delayedSuccess) {
                    // no session, i.e. late POST from the payment system. We must signal 200 OK.
                    header("HTTP/1.1 200 OK");
                    die('<html><head><meta http-equiv="refresh" content="0;url=' . Mage::getUrl('checkout/cart') . '"></head></html>');
                } else {
                    return $this->_redirect('checkout/onepage/success');
                }
            }
        } catch (Exception $e) {
            Mage::log(json_encode($e));
            $orderLocker->unlockOrder((int)$_request->getParam('s-f-1-36_order-number'));
        }

        return $this->_defaultAction();
    }

    public function indexAction()
    {
        return $this->getResponse()
            ->setHeader('Content-type', 'text/html; charset=utf8')
            ->setBody($this->getLayout()
                ->createBlock('VerifonePayment/redirect')
                ->setApiModel('VerifonePayment/api_paymentinterface_paymentInitialization')
                ->toHtml());
    }

    public function successAction()
    {
        $this->_handleSuccess(false);
    }

    public function successDelayedAction()
    {
        $this->_handleSuccess(true);
    }

    public function rejectedAction()
    {
        return $this->_defaultAction();
    }

    public function cancelAction()
    {
        return $this->_defaultAction();
    }

    public function expiredAction()
    {
        return $this->_defaultAction();
    }

    public function errorAction()
    {
        return $this->_defaultAction();
    }

    public function rechargeAction()
    {
        $profile_id = $this->getRequest()->getParam('profile_id');

        if (!Mage::getSingleton("customer/session")->isLoggedIn()) {
            $this->_redirect('customer/account/login');
        }

        if (!$profile_id) {
            $this->_redirect('/');
        }

        if ($profile_id) {
            $s2SGateModel = Mage::getModel('VerifonePayment/observer');
            $paymentsGateResp = $s2SGateModel->chargeSingleRecurringProfile($profile_id);
            $this->_redirect('customer/account');
        }
    }
}
