<?php

class Verifone_VerifonePayment_Model_Api_Paymentinterface_AddNewCard extends Verifone_VerifonePayment_Model_Api_Paymentinterface_Abstract
{
    protected $_paymentMethodCode;
    protected $_customer;

    public function _construct()
    {
        $this->setCustomer();
        $this->setPaymentMethodCode('');

        parent::_construct();
    }

    public function prepareData()
    {
        $_helper = Mage::helper('VerifonePayment');
        $_helperConfiguration = Mage::helper('VerifonePayment/configuration');

        $_requestTimestamp = gmdate('Y-m-d H:i:s');
        $_orderId = 'addNewCard';
        $_token = sprintf("%s;%s;%s", $_helperConfiguration->getMerchantAgreement(), $_orderId, $_requestTimestamp);
        $_customer = $this->getCustomer();

        if (!$_customer) {
            return parent::prepareData();
        }

        $_fields = array(
            's-f-32-32_payment-token' => strtoupper(substr(hash('sha256', $_token), 0, 32)),
            'locale-f-2-5_payment-locale' => Mage::app()->getLocale()->getLocaleCode(),
            't-f-14-19_payment-timestamp' => $_requestTimestamp,
            's-f-1-36_merchant-agreement-code' => $_helperConfiguration->getMerchantAgreement(),
            's-f-1-30_software' => 'Magento',
            's-f-1-10_software-version' => $_helper->_sanitise(Mage::getVersion(), 10, true),
            'i-f-1-11_interface-version' => '5',
            's-f-1-36_order-number' => $_orderId,
            's-t-1-36_order-note' => $this->getOrderNote(),
            't-f-14-19_order-timestamp' => $_requestTimestamp,
            'i-f-1-3_order-currency-code' => $_helper->convertCountryToISO4217(Mage::app()->getStore()->getCurrentCurrencyCode()),

            //'s-t-1-30_payment-method-code' => $this->getPaymentMethodCode(),
            'i-t-1-1_skip-confirmation-page' => Mage::getStoreConfig('payment/VerifonePayment/skip_confirmation_page'),
            's-t-1-30_style-code' => Mage::getStoreConfig('payment/VerifonePayment/style_code'),
            'i-t-1-1_save-payment-method' => '2',

            's-f-1-30_buyer-first-name' => substr($_customer->getFirstname(), 0, 30),
            's-f-1-30_buyer-last-name' => substr($_customer->getLastname(), 0, 30),
            's-f-1-100_buyer-email-address' => $_helper->_sanitise($_customer->getEmail(), 100),
            's-t-1-30_buyer-phone-number' => $this->getPhoneNumber(),

            'l-f-1-20_order-gross-amount' => '1',
            'l-f-1-20_order-net-amount' => '1',
            'l-f-1-20_order-vat-amount' => '0',
            'i-t-1-4_order-vat-percentage' => '0',
        );

        // Add billing address
        if ($_customer->getPrimaryBillingAddress() !== false) {
            $_billingAddress = $_customer->getPrimaryBillingAddress();
            $_fields['s-t-1-30_bill-to-address-line-one'] = $_helper->_sanitise($_billingAddress->getStreet1(), 30);
            $_fields['s-t-1-30_bill-to-address-line-two'] = $_helper->_sanitise($_billingAddress->getStreet2(), 30) ?
                $_helper->_sanitise($_billingAddress->getStreet2(), 30) : $_helper->_sanitise($_billingAddress->getRegion(), 30);
            $_fields['s-t-1-30_bill-to-address-line-three'] = $_helper->_sanitise($_billingAddress->getStreet2(),
                30) ? $_helper->_sanitise($_billingAddress->getRegion(), 30) : '';
            $_fields['s-t-1-30_bill-to-address-city'] = $_helper->_sanitise($_billingAddress->getCity(), 30);
            $_fields['s-t-1-30_bill-to-address-postal-code'] = $_helper->_sanitise($_billingAddress->getPostcode(), 30);
            $_fields['i-t-1-3_bill-to-address-country-code'] = $_helper->convertCountryCode2Numeric($_billingAddress->getCountry());
            $_fields['s-t-1-30_bill-to-address-first-name'] =  $_helper->_sanitise($_billingAddress->getFirstname(), 30);
            $_fields['s-t-1-30_bill-to-address-last-name'] =  $_helper->_sanitise($_billingAddress->getLastname(), 30);
            $_fields['s-t-1-30_bill-to-address-phone-number'] =  $_helper->_sanitise($_billingAddress->getTelephone(), 30);
            $_fields['s-t-1-100_bill-to-address-email'] =  $_helper->_sanitise($_billingAddress->getEmail(), 100);
        }

        $this->_fields = $_fields;

        return parent::prepareData();
    }

    public function setPaymentMethodCode($_code)
    {
        $this->_paymentMethodCode = $_code;
        return $this;
    }

    protected function setCustomer()
    {
        if (is_null($this->_customer)) {
            $session = Mage::getSingleton('customer/session');
            $_customer = Mage::getModel('customer/customer')->load($session->getId());

            $this->_customer = $_customer ? $_customer : null;
        }

        return $this;
    }

    /**
     * @return Mage_Customer_Model_Customer
     */
    public function getCustomer()
    {
        if (is_null($this->_customer)) {
            $this->setCustomer();
        }

        return $this->_customer;
    }

    protected function _getUrlsPIForm()
    {
        return Mage::helper('VerifonePayment/url')->getAddToCartReturnUrls();
    }

    protected function getPhoneNumber()
    {
        $_helper = Mage::helper('VerifonePayment');
        $session = Mage::getSingleton('customer/session');

        $_customer = $this->getCustomer();

        if (!$_customer || !$session->isLoggedIn()) {

            Mage::throwException($_helper->__('You need to be logged in to save Credit Card.'));

            return;
        }

        $_primaryBillingAddress = $_customer->getPrimaryBillingAddress();

        if ($_primaryBillingAddress && $_primaryBillingAddress->getTelephone()) {
            return $_helper->_sanitise($_primaryBillingAddress->getTelephone(), 30);
        }

        if ($_customer->getTelephone()) {
            return  $_helper->_sanitise($_customer->getTelephone(), 30);
        }

        Mage::throwException($_helper->__('Phone number is necessary to save Credit Card. Please update your default billing address.'));

        return;
    }

}
