<?php

class Verifone_VerifonePayment_Model_Api_S2s_ProcessPayment extends Verifone_VerifonePayment_Model_Api_S2s_Abstract
{
    protected $_order;
    protected $_quote;
    protected $_customer;
    protected $_savedPaymentMethodId;
    protected $_isRecurringPayment;

    const BASKET_LIMIT = 48;

    public function prepareData()
    {
        //$_customer = $this->getCustomer();
        $_order = $this->getOrder();
        $_helper = Mage::helper('VerifonePayment');
        $_helperConfiguration = Mage::helper('VerifonePayment/configuration');

        $_order_gross_amount = strval(round($_order->getGrandTotal(), 2) * 100);
        $_billingAddress = $_order->getBillingAddress();

        $_fields = array(
            's-f-1-30_operation' => 'process-payment', //this value is obligatory but it would be filled in every method
            't-f-14-19_request-timestamp' => gmdate('Y-m-d H:i:s'),
            's-f-1-36_merchant-agreement-code' => $_helperConfiguration->getMerchantAgreement(),
            's-f-1-30_software' => 'Magento',
            's-f-1-10_software-version' => $_helper->_sanitise(Mage::getVersion(), 10, true),
            'i-f-1-11_interface-version' => '5',
            'l-t-1-20_saved-payment-method-id' => $this->getSavedPaymentMethodId(),
            'locale-f-2-5_payment-locale' => Mage::getStoreConfig('general/locale/code', $_order->getStoreId()),
            't-f-14-19_payment-timestamp' => gmdate('Y-m-d H:i:s'),
            's-f-1-36_order-number' => $_order->getIncrementId(),
            't-f-14-19_order-timestamp' => $_order->getCreatedAt(),
            'i-f-1-3_order-currency-code' => $_helper->convertCountryToISO4217($_order->getOrderCurrencyCode()),
            'l-f-1-20_order-gross-amount' => $_order_gross_amount,

            's-f-1-30_buyer-first-name' => substr($_billingAddress->getFirstname(), 0, 30),
            's-f-1-30_buyer-last-name' => substr($_billingAddress->getLastname(), 0, 30),
            's-t-1-30_buyer-phone-number' => $_helper->_sanitise($_billingAddress->getTelephone(), 30),
            's-f-1-100_buyer-email-address' => $_helper->_sanitise($_billingAddress->getEmail(), 100),
        );

        if (Mage::getStoreConfig('payment/VerifonePayment/save_masked_pan_number')) {
            $_fields['s-t-1-1024_dynamic-feedback'] = 'i-t-2-2_card-pan-last2,i-t-6-6_card-pan-first6';
        }

        $this->setRequestId($_order->getIncrementId());

        if ($this->isRecurringPayment()) {
            $_fields['i-t-1-1_recurring-payment'] = '1';
            $_fields['s-t-1-30_recurring-payment-subscription-name'] = strtoupper(substr($_billingAddress->getFirstname() . ' ' . $_billingAddress->getLastname(), 0, 30));
            $_fields['s-t-1-30_recurring-payment-subscription-code'] = $_order->getProfileReferenceId();
        }

        if ($this->isSendBasketItems()) {
            $_fields = array_merge(
                $_fields,
                $this->getBasketItemsData()
            );
        }

        $this->_fields = $_fields;

        return parent::prepareData();
    }

    public function setSavedPaymentMethodId($_savedPaymentMethodId)
    {
        $this->_savedPaymentMethodId = $_savedPaymentMethodId;

        return $this;
    }

    protected function getSavedPaymentMethodId()
    {
        return $this->_savedPaymentMethodId ? $this->_savedPaymentMethodId : '';
    }

    public function setIsRecurringPayment($_isRecurringPayment)
    {
        $this->_isRecurringPayment = $_isRecurringPayment;

        return $this;
    }

    protected function isRecurringPayment()
    {
        return $this->_isRecurringPayment ? $this->_isRecurringPayment : '';
    }

    public function setQuote($_quote)
    {
        $this->_quote = $_quote;

        return $this;
    }

    protected function getQuote()
    {
        if (!$this->_quote) {
            $_session = Mage::getSingleton('checkout/session');
            $_quote = $_session->getQuote();
            $this->setQuote($_quote);
        }

        return $this->_quote;
    }

    public function setOrder($_order)
    {
        $this->_order = $_order;

        return $this;
    }

    protected function getOrder()
    {
        if (!$this->_order) {
            $_quote = $this->getQuote();
            $_order = $_quote ? $_quote->getOrder() : null;
            $this->setOrder($_order);
        }

        return $this->_order;
    }

    protected function getOrderGrossAmount()
    {
        return round($this->getOrder()->getGrandTotal(), 2) * 100;
    }

    protected function getOrderNetAmount()
    {
        return $this->getOrderGrossAmount() - $this->getOrderVatAmount();
    }

    protected function getOrderVatAmount()
    {
        return round($this->getOrder()->getTaxAmount(), 2) * 100;
    }

    protected function isSendBasketItems()
    {
        return ($this->isSendBasketItemsForInvoice() && $this->isMethodTypeInvoice()) || $this->isSendBasketItemsForAll();
    }

    /**
     * Verifone has an option to group basket items into 1 combined item.
     * Currently only possible to enable this for invoice payment methods.
     *
     * @return bool
     */
    protected function isCombineBasketItems()
    {
        return ($this->isCombineInvoiceBasketItems() && $this->isMethodTypeInvoice());
    }

    /**
     * If true, basket items are sent for all orders and all payment methods.
     *
     * @return bool
     */
    protected function isSendBasketItemsForAll()
    {
        $basketItemSending = Mage::getStoreConfig('payment/VerifonePayment/basket_item_sending');

        return $basketItemSending == Verifone_VerifonePayment_Model_Adminhtml_System_Config_Source_BasketItemSending::SEND_FOR_ALL;
    }

    /**
     * If true, basket items are sent for orders made with invoice payment methods.
     *
     * @return bool
     */
    protected function isSendBasketItemsForInvoice()
    {
        $basketItemSending = Mage::getStoreConfig('payment/VerifonePayment/basket_item_sending');

        return $basketItemSending == Verifone_VerifonePayment_Model_Adminhtml_System_Config_Source_BasketItemSending::SEND_FOR_INVOICE;
    }

    protected function isCombineInvoiceBasketItems()
    {
        return Mage::getStoreConfig('payment/VerifonePayment/combine_invoice_basket_items');
    }

    protected function getBasketItemsData()
    {
        $_fields = array();

        if (!$this->isSendBasketItems()) {
            return $_fields;
        }

        $i = $items_tax = $items_net_price = $items_gross_price = 0;
        $items = $this->getOrder()->getAllVisibleItems();

        if (count($items) >= self::BASKET_LIMIT) {
            $items = $this->groupItemsByVat($items);
        }

        /** @var Mage_Sales_Model_Order_Item $_item */
        foreach ($items as $_item) {
            $item_count = $_item->getQtyToInvoice() ? (int)$_item->getQtyToInvoice() : (int)$_item->getQtyOrdered();

            $_basketItemFields = $this->getBasketItemData($_item, $i, $item_count);

            Mage::dispatchEvent(
                'verifone_prepare_basket_item_after',
                array(
                    'storeId' => $this->getOrder()->getStoreId(),
                    'item' => $_item,
                    'basketItem' => &$_basketItemFields,
                    'count' => $i
                )
            );

            if (!$this->isCombineBasketItems()) {
                $_fields = array_merge(
                    $_fields,
                    $_basketItemFields
                );
            }

            $items_tax += $_basketItemFields["l-t-1-20_bi-gross-amount-$i"] - $_basketItemFields["l-t-1-20_bi-net-amount-$i"];
            $items_net_price += $_basketItemFields["l-t-1-20_bi-net-amount-$i"];
            $items_gross_price += $_basketItemFields["l-t-1-20_bi-gross-amount-$i"];

            $i++;
        }

        if ($this->getOrder()->getShippingAmount() != 0) {
            $shippingFields = $this->getBasketShippingAmountData($i);

            if (!$this->isCombineBasketItems()) {
                $_fields = array_merge(
                    $_fields,
                    $shippingFields
                );
            }

            $items_tax += $shippingFields["l-t-1-20_bi-gross-amount-$i"] - $shippingFields["l-t-1-20_bi-net-amount-$i"];
            $items_net_price += $shippingFields["l-t-1-20_bi-net-amount-$i"];
            $items_gross_price += $shippingFields["l-t-1-20_bi-gross-amount-$i"];
            $i++;
        }

        if (!$this->isCombineBasketItems()) {
            $discountAmount = $this->getOrderGrossAmount() - $items_gross_price;
            if (abs($discountAmount) >= 1) {
                $_fields = array_merge(
                    $_fields,
                    $this->getBasketDiscountData($i++, $this->getOrderGrossAmount(), $items_gross_price,
                        $this->getOrderNetAmount(), $items_net_price)
                );
            }
        }

        if ($this->isCombineBasketItems()) {
            $i = 0; // Reset items index, since we will only have 1 combined item.
            $_fields = array_merge(
                $_fields,
                $this->getCombinedBasketItemsData($i, $items_gross_price, $items_net_price)
            );
        }

        return $_fields;
    }

    /**
     * @param Mage_Sales_Model_Order_Item[] $items
     * @return Mage_Sales_Model_Order_Item[]
     */
    protected function groupItemsByVat($items)
    {
        /** @var Mage_Sales_Model_Order_Item[] $result */
        $result = array();

        foreach ($items as $item) {
            $tax = (string)$item->getTaxPercent();

            if (!isset($result[$tax])) {
                $mergedItem = new Mage_Sales_Model_Order_Item();
                $mergedItem->setName('Multiple items - vat ' . $tax);
                $mergedItem->setPrice(0);
                $mergedItem->setPriceInclTax(0);
                $mergedItem->setTaxPercent($item->getTaxPercent());
                $mergedItem->setQtyOrdered(1);

                $result[$tax] = $mergedItem;
            }

            $result[$tax]->setPrice($result[$tax]->getPrice() + $item->getRowTotal());
            $result[$tax]->setPriceInclTax($result[$tax]->getPriceInclTax() + $item->getRowTotalInclTax());
        }

        return $result;
    }

    /**
     * @param Mage_Sales_Model_Order_Item|Varien_Object $_item
     * @param int $i
     * @param int $item_count
     * @return array
     */
    protected function getBasketItemData($_item, $i, $item_count)
    {
        $_helper = Mage::helper('VerifonePayment');

        if ($_item->getProductType() == 'bundle') {
            $_children = $_item->getChildrenItems();
            $_taxes = array();
            foreach ($_children as $_child) {
                $_taxes[] = $_child->getTaxPercent();
            }

            $average = array_sum($_taxes) / count($_taxes);
            $_item->setTaxPercent($average);
        }

        return array(
            's-t-1-30_bi-name-' . $i => $_helper->_sanitise($_item->getName(), 30),
            'i-t-1-11_bi-unit-count-' . $i => $item_count,
            'l-t-1-20_bi-unit-cost-' . $i => round($_item->getPrice(), 2) * 100,
            'l-t-1-20_bi-net-amount-' . $i => round($_item->getPrice(), 2) * 100 * $item_count,
            'l-t-1-20_bi-gross-amount-' . $i => round($_item->getPriceInclTax(), 2) * 100 * $item_count,
            'i-t-1-4_bi-vat-percentage-' . $i => round($_item->getTaxPercent(), 2) * 100,
            'i-t-1-4_bi-discount-percentage-' . $i => 0,
        );
    }

    protected function getBasketShippingAmountData($i)
    {
        $_order = $this->getOrder();

        $_helper = Mage::helper('VerifonePayment');

        return array(
            's-t-1-30_bi-name-' . $i => $_helper->_sanitise($_order->getShippingDescription(), 30),
            'i-t-1-11_bi-unit-count-' . $i => 1,
            'l-t-1-20_bi-unit-cost-' . $i => round($_order->getShippingAmount(), 2) * 100,
            'l-t-1-20_bi-net-amount-' . $i => round($_order->getShippingAmount(), 2) * 100,
            'l-t-1-20_bi-gross-amount-' . $i => $_order->getShippingInclTax() ? round($_order->getShippingInclTax(), 2) * 100 : round($_order->getShippingAmount(), 2) * 100,
            'i-t-1-4_bi-vat-percentage-' . $i =>
                ($_order->getShippingInclTax() - $_order->getShippingAmount()) > 0 ? round(($_order->getShippingInclTax() - $_order->getShippingAmount()) / $_order->getShippingAmount(), 2) * 100 * 100 : 0,
            'i-t-1-4_bi-discount-percentage-' . $i => 0,
        );
    }

    protected function getBasketDiscountData($i, $order_gross_amount, $items_gross_price, $order_net_amount, $items_net_price)
    {
        $_helper = Mage::helper('VerifonePayment');

        return array(
            's-t-1-30_bi-name-' . $i => $_helper->_sanitise($this->getOrder()->getDiscountDescription(), 30),
            'i-t-1-11_bi-unit-count-' . $i => 1,
            'l-t-1-20_bi-unit-cost-' . $i => round($order_net_amount - $items_net_price, 0),
            'l-t-1-20_bi-net-amount-' . $i => round($order_net_amount - $items_net_price, 0),
            'l-t-1-20_bi-gross-amount-' . $i => round($order_gross_amount - $items_gross_price, 0),
            'i-t-1-4_bi-vat-percentage-' . $i =>
                round($order_net_amount - $items_net_price, 0) ? round((round($order_gross_amount - $items_gross_price, 0) - round($order_net_amount - $items_net_price, 0)) / round($order_net_amount - $items_net_price, 0), 2) * 100 * 100 : 0,
            'i-t-1-4_bi-discount-percentage-' . $i => 0,
        );
    }

    /**
     * @param $i
     * @param $order_gross_amount
     * @param $order_net_amount
     * @return array
     */
    protected function getCombinedBasketItemsData($i, $order_gross_amount, $order_net_amount)
    {
        $_helper = Mage::helper('VerifonePayment');
        /** @var $_order Mage_Sales_Model_Order */
        $_order = $this->getOrder();
        $incrementId = $_order->getIncrementId();
        // Must be "Tilaus %ORDERNUMBER%" according to docs
        $itemName = "Tilaus $incrementId";

        return array(
            's-t-1-30_bi-name-' . $i => $_helper->_sanitise($itemName, 30),
            'i-t-1-11_bi-unit-count-' . $i => $_order->getTotalItemCount(),
            'l-t-1-20_bi-unit-cost-' . $i => $order_net_amount,
            'l-t-1-20_bi-net-amount-' . $i => $order_net_amount,
            'l-t-1-20_bi-gross-amount-' . $i => $order_gross_amount,
            'i-t-1-4_bi-vat-percentage-' . $i => round(($order_gross_amount - $order_net_amount) / $order_net_amount, 2) * 100 * 100,
            'i-t-1-4_bi-discount-percentage-' . $i => 0,
        );
    }
}
