<?php

class Verifone_VerifonePayment_Model_Db_Pay_Options extends Mage_Core_Model_Abstract
{
    /**
     * Prefix of model events names
     *
     * @var string
     */
    protected $_eventPrefix = 'verifone_pay_options';

    protected function _construct()
    {
        $this->_init('VerifonePayment/db_pay_options');
    }

    protected $_options;

    public function toOptionArray($_cards = false)
    {
        if (!$this->_options) {
            $this->_options = array();
            $collection = $this->getCollection()->addFieldToFilter('active', array('eq' => '1'));

            $scope = $this->getCurrentScope();
            if ($scope['store'] != 0) {
                $collection->addFieldToFilter('scopes', array('finset' => array($scope['website'])));
            }
            if (!$_cards) {
                $collection->addFieldToFilter('type', array('neq' => 'CARD'));
            }

            foreach ($collection as $_item) {
                $this->_options[] = array('value' => $_item->getCode(), 'label' => Mage::helper('VerifonePayment')->__($_item->getName()));
            }
        }
        $options = $this->_options;
        return $options;
    }

    protected function getCurrentScope()
    {
        if (strlen($code = Mage::getSingleton('adminhtml/config_data')->getStore())) {
            // store level
            $websiteId = null;
            $storeId = Mage::getModel('core/store')->load($code)->getId();
        } elseif (strlen($code = Mage::getSingleton('adminhtml/config_data')->getWebsite())) {
            // website level
            $websiteId = Mage::getModel('core/website')->load($code)->getId();
            $storeId = Mage::app()->getWebsite($websiteId)->getDefaultStore()->getId();
        } else {
            // default level
            $websiteId = null;
            $storeId = 0;
        }

        return array(
            'website' => $websiteId,
            'store' => $storeId
        );
    }

    public function getCardsToOptionArray()
    {
        if (!$this->_options) {
            $this->_options = array();
            $collection = $this->getCollection()
                ->addFieldToFilter('active', array('eq' => '1'))
                ->addFieldToFilter('type', array('eq' => 'CARD'));

            $scope = $this->getCurrentScope();
            if ($scope['store'] != 0) {
                $collection->addFieldToFilter('scopes', array('finset' => array($scope['website'])));
            }

            foreach ($collection as $_item) {
                $this->_options[] = array('value' => $_item->getCode(), 'label' => Mage::helper('VerifonePayment')->__($_item->getName()));
            }
        }
        $options = $this->_options;
        return $options;
    }

    public function refreshOptions($gateWebsiteResponses)
    {
        $collection = $this->getCollection();

        if (isset($gateWebsiteResponses['s-f-1-30_error-message']) && $gateWebsiteResponses['s-f-1-30_error-message'] != '') {
            Mage::log($gateWebsiteResponses['s-f-1-30_error-message'], null, 'verifone-log.log');
            return $gateWebsiteResponses['s-f-1-30_error-message'];
        }

        foreach ($collection as $_item) {

            $_item->setActive('0');
            $itemWebsites = array();

            foreach ($gateWebsiteResponses as $key => $gateWebsiteResponse) {
                array_push($gateWebsiteResponse, 'VerifonePayment');

                if (in_array($_item->getCode(), $gateWebsiteResponse)) {
                    $_item->setActive('1');
                    $itemWebsites[] = $key;
                }
            }

            $_item->setData('scopes', implode(',', $itemWebsites));

            try {
                $_item->save();
            } catch (Exception $e) {
                return $e->getMessage();
            }
        }

        return Mage::helper('VerifonePayment')->__('Refresh succeed. Please refresh your web browser.');
    }

    public function getRecurringProfilesForPaymentProcess()
    {
        $now = Mage::getModel('core/date')->date('Y-m-d');
        $profiles = Mage::getModel('sales/recurring_profile')->getCollection()
            ->addFieldToFilter('state', array('eq' => Mage_Sales_Model_Recurring_Profile::STATE_ACTIVE))
            ->addFieldToFilter('method_code', array('like' => 'vp-recurring_%'))
            ->addFieldToFilter('updated_at', array('to' => $now))
            ->addFieldToFilter('start_datetime', array('to' => $now));

        $profiles->getSelect()
            ->where("
                            period_unit = 'day'         and NOW() >= DATE_ADD(updated_at, INTERVAL period_frequency DAY)
                        OR  period_unit = 'week'        and NOW() >= DATE_ADD(updated_at, INTERVAL period_frequency WEEK)
                        OR  period_unit = 'semi_month'  and NOW() >= DATE_ADD(updated_at, INTERVAL (period_frequency*2) WEEK)
                        OR  period_unit = 'month'       and NOW() >= DATE_ADD(updated_at, INTERVAL period_frequency MONTH)
                        OR  period_unit = 'year'        and NOW() >= DATE_ADD(updated_at, INTERVAL period_frequency YEAR)
                    ");

        return $profiles;
    }

    public function getRecurringProfilesForExpireCheck()
    {
        $now = Mage::getModel('core/date')->date('Y-m-d');
        $profiles = Mage::getModel('sales/recurring_profile')->getCollection()
            ->addFieldToFilter('state', array('eq' => Mage_Sales_Model_Recurring_Profile::STATE_ACTIVE))
            ->addFieldToFilter('method_code', array('like' => 'vp-recurring_%'))
            ->addFieldToFilter('updated_at', array('to' => $now))
            ->addFieldToFilter('start_datetime', array('to' => $now));

        return $profiles;
    }

    public function getRecurringProfileForPaymentProcess($profile_id)
    {
        return $profiles = Mage::getModel('sales/recurring_profile')->load($profile_id);
    }

    public function getAllVerifonePaymentMethods($_activeOnly = false)
    {
        $_collection = $this->getCollection();

        if ($_activeOnly) {
            $_collection->addFieldToFilter('active', array('eq' => '1'));
        }

        $_methods = array();
        foreach ($_collection as $_method) {
            $_methods[] = $_method->getCode() != 'VerifonePayment' ? 'vp_' . $_method->getCode() : $_method->getCode();

            if ($_method->getType() == 'CARD') {
                $_methods[] = 'vp-recurring_' . $_method->getCode();
            }
        }

        return $_methods;
    }
}