<?php

class Verifone_VerifonePayment_Model_Db_Pay_Saved extends Mage_Core_Model_Abstract
{
    protected $_customer;
    protected $_cards;

    const SAVE_METHOD_ORDER = 'order';
    const SAVE_METHOD_ADDNEWCARD= 'addNewCard';

    /**
     * Prefix of model events names
     *
     * @var string
     */
    protected $_eventPrefix = 'verifone_pay_saved';

    protected static $proceededCardRequests = array();

    protected function _construct()
    {
        $this->_init('VerifonePayment/db_pay_saved');
    }

    public function savePaymentMethod($_requestData, $_active = false)
    {
        $session = Mage::getSingleton('customer/session');

        if (!$session->getId()) {
            return;
        }

        if ($_customer = $this->getCustomer()) {
            $_savedMethod = $this
                ->setCustomerId($_customer->getId())
                ->setOrderId($_requestData['s-t-1-36_order-note'])
                ->setData('save_method', self::SAVE_METHOD_ADDNEWCARD)
                ->setActive('0');

            if (isset($_requestData['s-f-1-36_order-number']) && $_requestData['s-f-1-36_order-number'] != 'addNewCard') {
                $_savedMethod = $this->getSavedMethods($_requestData)
                    ->setCustomerId($_customer->getId())
                    ->setOrderId($_requestData['s-f-1-36_order-number'])
                    ->setData('save_method', self::SAVE_METHOD_ORDER)
                    ->setActive('0');
            }

            $_data = $this->filterRequestData($_requestData);

            $_savedMethod->setSerializedData(serialize($_data));

            return $_savedMethod->save();
        }

        $this->saveCardsInSession();

        return;
    }

    public function getSavedMethodsForCustomer($_activeOnly = false)
    {
        $_collection = $this->getCollection()
            ->addFieldToFilter('customer_id', array('eq' => $this->getCustomer()->getId()));

        if ($_activeOnly) {
            $_collection->addFieldToFilter('active', array('eq' => '1'));
        }

        return $_collection;
    }

    public function getSavedMethods($_requestData)
    {
        $_collection = $this->getSavedMethodsForCustomer()
            ->addFieldToFilter('customer_id', array('eq' => $this->getCustomer()->getId()));

        if (
            isset($_requestData['s-f-1-36_order-number'])
            && isset($_requestData['s-t-1-36_order-note'])
            && $_requestData['s-f-1-36_order-number'] == 'addNewCard'
        ) {

            $_collection->addFieldToFilter('order_id', array('eq' => $_requestData['s-t-1-36_order-note']));
        }

        if (
            isset($_requestData['s-f-1-36_order-number'])
            && $_requestData['s-f-1-36_order-number'] != 'addNewCard'
        ) {

            $_collection->addFieldToFilter('order_id', array('eq' => $_requestData['s-f-1-36_order-number']));
        }

        return $_collection->count() ? $_collection->getFirstItem() : $this;
    }

    protected function getCustomer()
    {
        if (is_null($this->_customer)) {
            $session = Mage::getSingleton('customer/session');
            $_customer = Mage::getModel('customer/customer')->load($session->getId());

            $this->_customer = $_customer ? $_customer : null;
        }

        return $this->_customer;
    }

    protected function isVerifoneDefaultCardId($_id)
    {
        $_customer = $this->getCustomer();
        $_defaultCardId = $_customer->getVerifoneDefaultCardId();

        return (boolean)($_defaultCardId == $_id);
    }

    protected function filterRequestData($_data)
    {
        $_result = array();

        //$_result['s-f-1-36_order-number'] = isset($_data['s-f-1-36_order-number']) ? $_data['s-f-1-36_order-number'] : '';
        $_result['s-f-1-30_buyer-first-name'] = isset($_data['s-f-1-30_buyer-first-name']) ? $_data['s-f-1-30_buyer-first-name'] : '';
        $_result['s-f-1-30_buyer-last-name'] = isset($_data['s-f-1-30_buyer-last-name']) ? $_data['s-f-1-30_buyer-last-name'] : '';
        $_result['s-f-1-100_buyer-email-address'] = isset($_data['s-f-1-100_buyer-email-address']) ? $_data['s-f-1-100_buyer-email-address'] : '';
        $_result['s-t-1-30_buyer-phone-number'] = isset($_data['s-t-1-30_buyer-phone-number']) ? $_data['s-t-1-30_buyer-phone-number'] : '';
        $_result['s-t-1-30_delivery-address-line-one'] = isset($_data['s-t-1-30_delivery-address-line-one']) ? $_data['s-t-1-30_delivery-address-line-one'] : '';
        $_result['s-t-1-30_delivery-address-line-two'] = isset($_data['s-t-1-30_delivery-address-line-two']) ? $_data['s-t-1-30_delivery-address-line-two'] : '';
        $_result['s-t-1-30_delivery-address-line-three'] = isset($_data['s-t-1-30_delivery-address-line-three']) ? $_data['s-t-1-30_delivery-address-line-three'] : '';
        $_result['s-t-1-30_delivery-address-city'] = isset($_data['s-t-1-30_delivery-address-city']) ? $_data['s-t-1-30_delivery-address-city'] : '';
        $_result['s-t-1-30_delivery-address-postal-code'] = isset($_data['s-t-1-30_delivery-address-postal-code']) ? $_data['s-t-1-30_delivery-address-postal-code'] : '';
        $_result['i-t-1-3_delivery-address-country-code'] = isset($_data['i-t-1-3_delivery-address-country-code']) ? $_data['i-t-1-3_delivery-address-country-code'] : '';

        return array_filter($_result, 'strlen');
    }

    public function getSavedPayments($_forceRequest = false)
    {
        if (!is_null($this->getCardsFromSession()) && !$_forceRequest) {
            return $this->getCardsFromSession();
        }
        $_savedCardsCollection = $this->getSavedMethodsForCustomer($_active = true);

        $_result = array();

        foreach ($_savedCardsCollection as $_card) {
            foreach ($_card->getCardDetails() as $_key => $_savedCard) {
                $_result[$_key] = $_savedCard;
                $_result[$_key]['is_default'] = $this->isVerifoneDefaultCardId($_key);

                if (!$this->getCardByGateMethodId($_key)->getId()) {

                    $_card
                        ->setId(null)
                        ->setCustomerId($this->getCustomer()->getId())
                        ->setActive(1)
                        ->setGateMethodId($_key)
                        ->save();
                }
            }
        }

        return $this->saveCardsInSession($_result);
    }

    public function getCardDetails()
    {
        $_additionalData = unserialize($this->getSerializedData());

        $_apiListSavedPaymentMethods = Mage::getModel('VerifonePayment/api_s2s_listSavedPaymentMethods');

        $_savedPaymentMethodsResponse = $_apiListSavedPaymentMethods->setAdditionalData($_additionalData)
            ->sendRequest()
            ->getFormatedResponse();

        $_formattedSavedPaymentMethods = $this->formatSavedPaymentMethods($_savedPaymentMethodsResponse);

        if (empty($_formattedSavedPaymentMethods)) {

            $this->load($this->getId());
            $this->delete();
        }

        return $_formattedSavedPaymentMethods;

    }

    public function updatePaymentMethods()
    {
        $requestsToCheck = $this->getNotActivatedMethodsForCustomer();
        if(count($requestsToCheck) > 0) {
            foreach ($requestsToCheck as $request) {
                if (!in_array($request->getId(), self::$proceededCardRequests)) {
                    self::$proceededCardRequests[] = $request->getId();
                    foreach ($request->getCardDetails() as $_key => $_savedCard) {
                        $_cardObj = $this->getCardByGateMethodId($_key);

                        $_cardObj
                            ->setSerializedData($request->getSerializedData())
                            ->setCustomerId($this->getCustomer()->getId())
                            ->setActive(1)
                            ->setGateMethodId($_key)
                            ->save();

                        unset($_cardObj);
                    }
                } else {
                    return true;
                }
            }
            return $this->savedCleanUp();
        }

        return true;
    }

    public function savedCleanUp()
    {
        $_collection = $this->getNotActivatedMethodsForCustomer();

        foreach ($_collection as $_inactiveCard) {
            if ($_inactiveCard->getId()) {
                $_inactiveCard
                    ->load($_inactiveCard->getId())
                    ->delete();
            }
        }

        return $this;
    }

    public function getNotActivatedMethodsForCustomer()
    {
        $_collection = $this->getCollection()
            ->addFieldToFilter('customer_id', array('eq' => $this->getCustomer()->getId()))
            ->addFieldToFilter('active', 0);

        return $_collection;
    }

    public function getCardByOrderId($_orderId)
    {
        if (!$_orderId) {
            return;
        }

        $_collection = $this->getSavedMethodsForCustomer();

        $_collection->addFieldToFilter('order_id', array('eq' => $_orderId));

        return $_collection->getFirstItem();
    }

    public function getCardByGateMethodId($_gateMethodId)
    {
        $_collection = $this->getSavedMethodsForCustomer();

        $_collection->addFieldToFilter('gate_method_id', array('eq' => $_gateMethodId));

        $_card = $_collection->getFirstItem();

        if (!$_card) {
            $_card = Mage::getModel('VerifonePayment/db_pay_saved');
        }

        return $_card;
    }

    public function removeByGateMethodId($_gateMethodId)
    {
        $_cardToRemove = $this->getCardByGateMethodId($_gateMethodId);

        if ($_cardToRemove->load($_cardToRemove->getId())) {
            return $_cardToRemove->delete();
        }

        return;
    }

    protected function saveCardsInSession($_cards = null)
    {
        $_session = Mage::getSingleton('customer/session');
        $_session->setVerifoneCards($_cards);

        return $_cards;
    }

    protected function getCardsFromSession()
    {
        $_session = Mage::getSingleton('customer/session');
        return $_session->getVerifoneCards();
    }

    protected function formatSavedPaymentMethods($_response = array())
    {
        if (empty($_response)) {
            return array();
        }

        $_result = array();

        foreach ($_response as $_key => $_field) {

            if (strpos($_key, 'l-t-1-20_payment-method-id-') !== false) {
                $_id = (int)str_replace('l-t-1-20_payment-method-id-', '', $_key);
                $_result[$_id]['l-t-1-20_payment-method-id'] = $_field;
            }

            if (strpos($_key, 's-t-1-30_payment-method-code-') !== false) {
                $_id = (int)str_replace('s-t-1-30_payment-method-code-', '', $_key);
                $_result[$_id]['s-t-1-30_payment-method-code'] = $_field;
            }

            if (strpos($_key, 's-t-1-30_payment-method-title-') !== false) {
                $_id = (int)str_replace('s-t-1-30_payment-method-title-', '', $_key);
                $_result[$_id]['s-t-1-30_payment-method-title'] = $_field;
            }

            if (strpos($_key, 's-t-1-6_card-expected-validity-') !== false) {
                $_id = (int)str_replace('s-t-1-6_card-expected-validity-', '', $_key);
                $_result[$_id]['s-t-1-6_card-expected-validity'] = $_field;
            }
        }

        return $this->formatSavedPaymentsResultArray($_result);

    }

    protected function formatSavedPaymentsResultArray($_savedPaymentsArray = array())
    {
        if (empty($_savedPaymentsArray)) {
            return array();
        }

        $_result = array();
        foreach ($_savedPaymentsArray as $_savedPayment) {
            $_result[$_savedPayment['l-t-1-20_payment-method-id']] = $_savedPayment;
        }

        return $_result;
    }

}
