<?php

/**
 * Class Nets_Payment_Model_Observer
 *
 * -> events            in Verifone_VerifonePayment_Model_Observer_Events
 * -> cron jobs         in Verifone_VerifonePayment_Model_Observer_Cron
 */
class Verifone_VerifonePayment_Model_Observer extends Mage_Core_Model_Abstract
{

    public function chargeSingleRecurringProfile($profile_id)
    {
        $db = Mage::getModel('VerifonePayment/db_pay_options');
        $profile = $db->getRecurringProfileForPaymentProcess($profile_id);
        $return = false;

        if ($profile) {
            $recurring = $this->getMethodNamespace($profile->getMethodCode());
            $orders = $profile->getResource()->getChildOrderIds($profile);
            $countBillingCycling = count($orders);

            if ($recurring->chargeRecurringProfile($profile, true)) {
                $countBillingCycling++;
                $return = true;
            } else {
                $storeId = $profile->getStoreId();
                $templateId = Mage::getStoreConfig('payment/VerifonePayment/payment_failed_email_template', $storeId);
                $email_to = $profile->getOrderInfo()['customer_email'];
                $customer_name = $profile->getOrderInfo()['customer_firstname'] . ' ' . $profile->getOrderInfo()['customer_lastname'];
                $email_template = Mage::getModel('core/email_template')->loadDefault($templateId);

                $email_template_variables = array(
                    'customer_name' => $customer_name,
                    'order_no' => Mage::getModel('sales/order')->load(end($orders))->getIncrementId(),
                    'amount' => $profile->getBillingAmount() + $profile->getShippingAmount() + $profile->getTaxAmount(),
                    'date' => Mage::getModel('core/date')->date('Y.m.d'),
                );

                $sender_name = Mage::getStoreConfig(Mage_Core_Model_Store::XML_PATH_STORE_STORE_NAME);
                $sender_email = Mage::getStoreConfig('trans_email/ident_general/email');
                $email_template->setSenderName($sender_name);
                $email_template->setSenderEmail($sender_email);

                $email_template->send($email_to, $customer_name, $email_template_variables);
            }

            if ($profile->getPeriodMaxCycles() && $countBillingCycling >= $profile->getPeriodMaxCycles()) {
                $profile->setState(Mage_Sales_Model_Recurring_Profile::STATE_SUSPENDED);
                $profile->save();
            }
        }
        return $return;
    }

    protected function formatProfile($profile)
    {
        $profile->setOrderInfo(unserialize($profile->getOrderInfo()));
        $profile->setOrderItemInfo(unserialize($profile->getOrderItemInfo()));
        $profile->setBillingAddressInfo(unserialize($profile->getBillingAddressInfo()));
        $profile->setShippingAddressInfo(unserialize($profile->getShippingAddressInfo()));

        return $profile;
    }

    protected function getMethodNamespace($code)
    {
        $allAvailablePaymentMethods = Mage::getModel('payment/config')->getAllMethods();
        foreach ($allAvailablePaymentMethods as $method) {
            if ($method->getId() == $code) {
                return $method;
            }
        }
    }

    /**
     * @return Mage_Sales_Model_Resource_Order_Collection
     */
    protected function getOrdersToCheck()
    {
        $_db = Mage::getModel('VerifonePayment/db_pay_options');

        $_ordersCollection = Mage::getModel('sales/order')->getCollection();

        $_ordersCollection->getSelect()
            ->joinLeft(
                array('sfop' => Mage::getSingleton('core/resource')->getTableName('sales/order_payment')),
                'sfop.parent_id=main_table.entity_id',
                array('method')
            );

        $_ordersCollection
            ->addFieldToFilter('status', array('in' => array('canceled', 'pending_payment')))
            ->addFieldToFilter('method', array('in' => $_db->getAllVerifonePaymentMethods()));

        return $_ordersCollection;
    }

}
