<?php


class Verifone_VerifonePayment_Model_Payment_Recurring extends Verifone_VerifonePayment_Model_Payment
    implements Mage_Payment_Model_Recurring_Profile_MethodInterface
{
    protected $_code = 'vp_recurring';

    public function canUseCheckout()
    {
        $cart = Mage::getModel('checkout/cart')->getQuote();
        foreach ($cart->getAllItems() as $item) {
            if (!$item->getProduct()->getIsRecurring())
                return false;
        }
        return true;
    }

    /**
     * Validate data
     *
     * @param Mage_Payment_Model_Recurring_Profile $profile
     * @throws Mage_Core_Exception
     */
    public function validateRecurringProfile(Mage_Payment_Model_Recurring_Profile $profile)
    {
        return $this;
    }

    /**
     * Submit to the gateway
     *
     * @param Mage_Payment_Model_Recurring_Profile $profile
     * @param Mage_Payment_Model_Info $paymentInfo
     */
    public function submitRecurringProfile(Mage_Payment_Model_Recurring_Profile $profile, Mage_Payment_Model_Info $paymentInfo)
    {
        $referenceId = $this->getVerifoneToken();

        $this->getCheckout()->setRecurringProfileReferenceId($referenceId);

        $profile->setReferenceId($referenceId);

        $paymentInfo->setSkipTransactionCreation(true);

        // send fake request to charge
        $this->chargeRecurringProfile($profile);
        return $this;
    }

    /**
     * Fetch details
     *
     * @param string $referenceId
     * @param Varien_Object $result
     */
    public function getRecurringProfileDetails($referenceId, Varien_Object $result)
    {
        return $this;
    }

    /**
     * Check whether can get recurring profile details
     *
     * @return bool
     */
    public function canGetRecurringProfileDetails()
    {
        return true;
    }

    /**
     * Update data
     *
     * @param Mage_Payment_Model_Recurring_Profile $profile
     */
    public function updateRecurringProfile(Mage_Payment_Model_Recurring_Profile $profile)
    {
        return $this;
    }

    /**
     * Manage status
     *
     * @param Mage_Payment_Model_Recurring_Profile $profile
     */
    public function updateRecurringProfileStatus(Mage_Payment_Model_Recurring_Profile $profile)
    {
        switch ($profile->getNewState()) {
            case Mage_Sales_Model_Recurring_Profile::STATE_ACTIVE:
                $action = 'start';
                break;
            case Mage_Sales_Model_Recurring_Profile::STATE_CANCELED:
                $action = 'cancel';
                break;
            case Mage_Sales_Model_Recurring_Profile::STATE_EXPIRED:
                $action = 'cancel';
                break;
            case Mage_Sales_Model_Recurring_Profile::STATE_SUSPENDED:
                $action = 'stop';
                break;
            default:
                return $this;
        }
        $additionalInfo = $profile->getAdditionalInfo() ? $profile->getAdditionalInfo() : array();


        if ($action == 'start') {
            $profile->setUpdatedAt(date('Y-m-d H:i:s'));
            $profile->save();
        }
        return $this;

    }

    public function chargeRecurringProfile(Mage_Payment_Model_Recurring_Profile $profile, $cronjob = false)
    {
        $productItemInfo = new Varien_Object;
        $productItemInfo->setPaymentType(Mage_Sales_Model_Recurring_Profile::PAYMENT_TYPE_REGULAR);
        $productItemInfo->setPrice($profile->getBillingAmount());

        $_order = $profile->createOrder($productItemInfo);
        $_order->setState(Mage_Sales_Model_Order::STATE_NEW);
        $_order->setStatus(Mage_Sales_Model_Order::STATE_PENDING_PAYMENT);
        $_order->setQuoteId($this->getCheckout()->getQuote()->getId());
        $_order->save();

        $_quotePayment = $this->getCheckout()->getQuote()->getPayment();
        $_orderPayment = $_order->getPayment()
            ->setQuoteId($_quotePayment->getQuoteId())
            ->setAdditionalInformation($_quotePayment->getAdditionalInformation())
            ->save();

        $profile->addOrderRelation($_order->getId());

        if ($cronjob) {
            $_order->setProfileReferenceId($profile->getReferenceId());

            $_apiListSavedPaymentMethods = Mage::getModel('VerifonePayment/api_s2s_listSavedPaymentMethods');
            $_savedPaymentMethodsResponse = $_apiListSavedPaymentMethods->setOrder($_order)
                ->sendRequest()
                ->getFormatedResponse();

            $paymentMethodId = Mage::helper('VerifonePayment')->getPaymentMethodCode($_savedPaymentMethodsResponse, $_order->getPayment()->getMethod());

            $_apiProcessPayment = Mage::getModel('VerifonePayment/api_s2s_processPayment');

            $_processPaymentResponse = $_apiProcessPayment->setSavedPaymentMethodId($paymentMethodId)
                ->setOrder($_order)
                ->sendRequest()
                ->getFormatedResponse();

            if (empty($_processPaymentResponse)) {
                $_order->setStatus(Mage_Sales_Model_Order::STATE_CANCELED);
                $_order->save();
                return false;
            } else {
                $_order->setState(Mage_Sales_Model_Order::STATE_PROCESSING);
            }
        }

        $profile->setState(Mage_Sales_Model_Recurring_Profile::STATE_ACTIVE);
        // change updated_at to one cycle ahead
        $this->_setUpdateDateToNextPeriod($profile->getId());
        return true;
    }

    protected function _setUpdateDateToNextPeriod($profile_id)
    {
        $_resource = Mage::getSingleton('core/resource');
        $sql = '
			UPDATE ' . $_resource->getTableName('sales_recurring_profile') . '
			SET updated_at = CASE period_unit
				WHEN "day" 			THEN DATE_ADD(updated_at, INTERVAL period_frequency DAY)
				WHEN "week" 		THEN DATE_ADD(updated_at, INTERVAL (period_frequency*7) DAY)
				WHEN "semi_month" 	THEN DATE_ADD(updated_at, INTERVAL (period_frequency*14) DAY)
				WHEN "month" 		THEN DATE_ADD(updated_at, INTERVAL period_frequency MONTH)
				WHEN "year" 		THEN DATE_ADD(updated_at, INTERVAL period_frequency YEAR)
			END
			WHERE profile_id = :pid';

        $connection = $_resource->getConnection('core_write');
        $pdoStatement = $connection->prepare($sql);
        $pdoStatement->bindValue(':pid', $profile_id);
        return $pdoStatement->execute();
    }

    public function getGateCode($code = null)
    {
        if ($code) {
            return substr($code, 13);
        }

        return substr($this->getCode(), 13);
    }

    protected function getVerifoneToken()
    {
        return 'verifone-' . uniqid();
    }
}